//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uWaitThread.pas
//              seperate thread, waiting for interrupt to arrive
//==============================================================================
//
unit uWaitThread;

interface

uses
  Classes, Windows, Messages, Vcl.Forms;

type
  TWaitThread = class(TThread)
  private
    { Private declarations }
    pvContext: Pointer;
    idxDevice: Integer;
    idxSubdevice: Integer;
    iTimeout: Integer;
  protected
    procedure Execute; override;
  public
    constructor Create(iDevice: Integer; iSubDevice: Integer; context: Pointer; timeout: Integer);
  end;


implementation

uses meError, meIDS, uMain;

constructor TWaitThread.Create(iDevice, iSubDevice: Integer; context: Pointer;
  timeout: Integer);
begin
  inherited Create(true);
  idxDevice := iDevice;
  idxSubdevice := iSubDevice;
  pvContext := context;
  iTimeout := timeout;

end;

procedure TWaitThread.Execute;
var
  irqCount: Integer;
  irqValue: Integer;
  meError: Integer;
begin
  FreeOnTerminate := True;

  while True do
  begin
    if Terminated then Break;

    meError := meIOIrqWait(idxDevice, idxSubdevice, ME_VALUE_NOT_USED, irqCount, irqValue, iTimeout, ME_IO_IRQ_WAIT_NO_FLAGS);
    if meError = ME_ERRNO_SUCCESS then
    begin
      meIOIrqCB(idxDevice, idxSubdevice, 0, irqCount, irqValue, pvContext, meError);
    end else
    begin
      PostMessage(TForm(pvContext).Handle, WM_IRQ_WAIT_ERROR, 0, meError);
      Break;
    end;
  end;

  meIOIrqStop(idxDevice, idxSubdevice, 0, ME_IO_IRQ_STOP_NO_FLAGS);
end;

end.
