/*
 * Copyright (C) 2005 Meilhaus Electronic GmbH (support@meilhaus.de)
 *
 * Source File : medrivervv.h                                              
 * Author      : GG (Guenter Gebhardt)  <g.gebhardt@meilhaus.de>
 * adapted to vee: RF
 */

	/*===========================================================================
	  Functions to access the driver system
	  =========================================================================*/

	int meOpen(int iFlags);
	int meClose(int iFlags);

	int meLockDriver(int iLock, int iFlags);
	int meLockDevice(int iDevice, int iLock, int iFlags);
	int meLockSubdevice(int iDevice, int iSubdevice, int iLock, int iFlags);

	/*===========================================================================
	  Error handling functions
	  =========================================================================*/

	int meErrorGetLastMessage(char *pcErrorMsg, int iCount);
	int meErrorGetMessage(int iErrorCode, char *pcErrorMsg, int iCount);
//	int meErrorSetDefaultProc(int iSwitch);
//	int meErrorSetUserProc(meErrorCB_t pErrorProc);


	/*===========================================================================
	  Functions to perform I/O on a device
	  =========================================================================*/

/*	int meIOIrqSetCallback(
			int iDevice,
			int iSubdevice,
			meIOIrqCB_t pCallback,
			void *pCallbackContext,
			int iFlags);*/
	int meIOIrqStart(
			int iDevice,
			int iSubdevice,
			int iChannel,
			int iIrqSource,
			int iIrqEdge,
			int iIrqArg,
			int iFlags);
	int meIOIrqStop(
			int iDevice,
			int iSubdevice,
			int iChannel,
			int iFlags);
	int meIOIrqWait(
			int iDevice,
			int iSubdevice,
			int iChannel,
			int *piIrqCount,
			int *piValue,
			int iTimeOut,
			int iFlags);

	int meIOResetDevice(int iDevice, int iFlags);
	int meIOResetSubdevice(int iDevice, int iSubdevice, int iFlags);

	int meIOStreamFrequencyToTicks(
			int iDevice,
			int iSubdevice,
			int iTimer,
			double *pdFrequency,
			int *piTicksLow,
			int *piTicksHigh,
			int iFlags);

	int meIOSingleConfig(
			int iDevice,
			int iSubdevice,
			int iChannel,
			int iSingleConfig,
			int iRef,
			int iTrigChan,
			int iTrigType,
			int iTrigEdge,
			int iFlags);
	int meIOSingle(int *lpSingleList, int iCount, int iFlags);

	int meIOStreamConfig(
			int iDevice,
			int iSubdevice,
			int *lpConfigList,
			int iCount,
			int *lpTrigger,
			int iFifoIrqThreshold,
			int iFlags);
	int meIOStreamNewValues(
			int iDevice,
			int iSubdevice,
			int iTimeOut,
			int *piCount,
			int iFlags);
	int meIOStreamRead(
			int iDevice,
			int iSubdevice,
			int iReadMode,
			int *piValues,
			int *piCount,
			int iFlags);
	int meIOStreamWrite(
			int iDevice,
			int iSubdevice,
			int iWriteMode,
			int *piValues,
			int *piCount,
			int iFlags);
	int meIOStreamStart(int *lpStartList, int iCount, int iFlags);
	int meIOStreamStop(int *lpStopList, int iCount, int iFlags);
	int meIOStreamStatus(
			int iDevice,
			int iSubdevice,
			int iWait,
			int *piStatus,
			int *piCount,
			int iFlags);
/*	int meIOStreamSetCallbacks(
			int iDevice,
			int iSubdevice,
			meIOStreamCB_t pStartCB,
			void *pStartCBContext,
			meIOStreamCB_t pNewValuesCB,
			void *pNewValuesCBContext,
			meIOStreamCB_t pEndCB,
			void *pEndCBContext,
			int iFlags);*/
			
	int meIOStreamTimeToTicks(
			int iDevice,
			int iSubdevice,
			int iTimer,
			double *pdTime,
			int *piTicksLow,
			int *piTicksHigh,
			int iFlags);


	/*===========================================================================
	  Functions to query the driver system
	  =========================================================================*/

	int meQueryDescriptionDevice(int iDevice, char *pcDescription, int iCount);

	int meQueryInfoDevice(
			int iDevice,
			int *piVendorId,
			int *piDeviceId,
			int *piSerialNo,
			int *piBusType,
			int *piBusNo,
			int *piDevNo,
			int *piFuncNo,
			int *piPlugged);

	int meQueryNameDevice(int iDevice, char *pcName, int iCount);
	int meQueryNameDeviceDriver(int iDevice, char *pcName, int iCount);

	int meQueryNumberDevices(int *piNumber);
	int meQueryNumberSubdevices(int iDevice, int *piNumber);
	int meQueryNumberChannels(int iDevice, int iSubdevice, int *piNumber);
	int meQueryNumberRanges(
			int iDevice,
			int iSubdevice,
			int iUnit,
			int *piNumber);
	int meQueryRangeByMinMax(
			int iDevice,
			int iSubdevice,
			int iUnit,
			double *pdMin,
			double *pdMax,
			int *piMaxData,
			int *piRange);
	int meQueryRangeInfo(
			int iDevice,
			int iSubdevice,
			int iRange,
			int *piUnit,
			double *pdMin,
			double *pdMax,
			int *piMaxData);
	int meQuerySubdeviceByType(
			int iDevice,
			int iStartSubdevice,
			int iType,
			int iSubtype,
			int *piSubdevice);
	int meQuerySubdeviceType(
			int iDevice,
			int iSubdevice,
			int *piType,
			int *piSubtype);
	int meQuerySubdeviceCaps(
			int iDevice,
			int iSubdevice,
			int *piCaps);
	int meQuerySubdeviceCapsArgs(
			int iDevice,
			int iSubdevice,
			int iCap,
			int *piArgs,
			int iCount);

	int meQueryVersionLibrary(int *piVersion);
	int meQueryVersionMainDriver(int *piVersion);
	int meQueryVersionDeviceDriver(int iDevice, int *piVersion);


	/*===========================================================================
	  Common utility functions
	  =========================================================================*/

	int meUtilityExtractValues(
			int iChannel,
			int *piAIBuffer,
			int iAIBufferCount,
			int *lpConfigList,
			int iConfigListCount,
			int *piChanBuffer,
			int *piChanBufferCount);
	int meUtilityDigitalToPhysical(
			double dMin,
			double dMax,
			int iMaxData,
			int iData,
			int iModuleType,
			double dRefValue,
			double *pdPhysical);
	int meUtilityDigitalToPhysicalV(
			double dMin,
			double dMax,
			int iMaxData,
			int *piDataBuffer,
			int iCount,
			int iModuleType,
			double dRefValue,
			double *pdPhysicalBuffer);
	int meUtilityPhysicalToDigital(
			double dMin,
			double dMax,
			int iMaxData,
			double dPhysical,
			int *piData);
	int meUtilityPhysicalToDigitalV(
			double dMin,
			double dMax,
			int iMaxData,
			double* pdPhysicalBuffer,
			int iCount,
			int *piDataBuffer);
	int meUtilityPWMStart(
			int iDevice,
			int iSubdevice,
			int iRef,
			int iPrescaler,
			int iDutyCycle);
	int meUtilityPWMStop(int iDevice, int iSubdevice);


	/*===========================================================================
	  Load configuration from file into driver system
	  =========================================================================*/

	int meConfigLoad(char *pcConfigFile);


