﻿'==============================================================================
'Copyright(c), Meilhaus Electronic GmbH
'              http://www.meilhaus.de
'              support@meilhaus.de
'
'Source File : FormMain.vb
'Description : Example program for reading property values
'              - Open main driver (ME-iDS)
'              - Create the root node (ME-iDS system container) in a TreeView
'              - Go recursively through the property containers and display all properties found
'              - add the name and the value of the property to the node text 
'              - add property attributes depending on the data type of the property 
'              - Close main driver (ME-iDS) when program is closed
'              - Properties can only be read from devices that are "plugged in"
'==============================================================================
Imports System.Text

Public Class frmMain
    Private openErrSuccess As Integer = Constants.ME_IDS_NOTOPENED

    Private Sub frmMain_Shown(sender As System.Object, e As System.EventArgs) Handles MyBase.Shown
        Try
            openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS)
            If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
                DisplayMEiDSProperties()
            End If
        Catch ex As DllNotFoundException
            MessageBox.Show("The ME-iDS is not properly installed." & Environment.NewLine & ex.Message, "ME-iDS Properties")
        End Try
    End Sub

    Private Sub DisplayMEiDSProperties()
        Dim treeNode = New TreeNode(meIDS.ME_CONTAINER_NAME_ROOT)
        tv1.Nodes.Add(treeNode)
        DisplayContainerProperties(treeNode, "\")
        tv1.ExpandAll()
        tv1.SelectedNode = treeNode
        tv1.Focus()
    End Sub

    Private Sub DisplayContainerProperties(ByRef parentNode As TreeNode, containerPath As String)
        Dim propertyPath As String = containerPath + "\" + meIDS.ME_ATTRIBUTE_NAME_NUMBER_OF_ELEMENTS
        Dim numElements As Integer
        Dim propTypeVal As Integer

        Dim propStrVal = New StringBuilder(Constants.PROPERTY_STRING_LEN)
        Dim meError = meIDS.mePropertyGetIntW(propertyPath, numElements)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            Dim i As Integer
            For i = 0 To numElements - 1
                Dim propName = New StringBuilder()
                propName.Append(containerPath & "\" & i.ToString)
                meError = meIDS.mePropertyGetStringW(propName.ToString & "\" & meIDS.ME_ATTRIBUTE_NAME_PROPERTY_NAME, propStrVal, propStrVal.Capacity)
                If meError = meIDS.ME_ERRNO_SUCCESS Then
                    Dim propType = New StringBuilder(containerPath & "\" & i.ToString & "\" & meIDS.ME_ATTRIBUTE_NAME_PROPERTY_TYPE)
                    meError = meIDS.mePropertyGetIntW(propType.ToString(), propTypeVal)
                    If meError = meIDS.ME_ERRNO_SUCCESS Then
                        Dim nodeText = New StringBuilder(propStrVal.ToString())
                        If propTypeVal = meIDS.ME_PROPERTY_TYPE_CONTAINER Then
                            nodeText.Append(" Type: Container")
                            Dim treeNodeContainer = New TreeNode(nodeText.ToString())
                            parentNode.Nodes.Add(treeNodeContainer)
                            DisplayContainerProperties(treeNodeContainer, containerPath & "\" & propStrVal.ToString)
                        Else
                            Dim propAccType As Integer
                            Dim propAccess = New StringBuilder(containerPath.ToString & "\" & propStrVal.ToString & "\" & meIDS.ME_ATTRIBUTE_NAME_PROPERTY_ACCESS)
                            meError = meIDS.mePropertyGetIntW(propAccess.ToString(), propAccType)
                            If meError = meIDS.ME_ERRNO_SUCCESS Then
                                nodeText.Append(GetAccessTypeString(propAccType))
                            End If

                            Dim propValue = New StringBuilder(containerPath & "\" & propStrVal.ToString())
                            If propTypeVal = meIDS.ME_PROPERTY_TYPE_BOOL Then
                                Dim propBoolVal As Integer
                                nodeText.Append(", Type: Boolean")
                                meError = meIDS.mePropertyGetIntW(propValue.ToString(), propBoolVal)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then
                                    nodeText.Append(", Value: " & GetPropertyBoolString(propBoolVal))
                                End If
                            ElseIf propTypeVal = meIDS.ME_PROPERTY_TYPE_INT Then
                                Dim propIntVal As Integer
                                nodeText.Append(", Type: Integer")
                                meError = meIDS.mePropertyGetIntW(propValue.ToString(), propIntVal)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then
                                    nodeText.Append(String.Format(", Value: {0} (0x{0:X})", propIntVal, propIntVal))
                                End If

                                If propAccType = meIDS.ME_PROPERTY_ACCESS_READ_WRITE Then
                                    Dim propIntMin As Integer
                                    Dim propIntMax As Integer
                                    Dim propIntMinVal = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_MIN_VALUE)
                                    meError = meIDS.mePropertyGetIntW(propIntMinVal.ToString(), propIntMin)
                                    If meError = meIDS.ME_ERRNO_SUCCESS Then
                                        nodeText.Append(", Minimum: " & propIntMin.ToString())
                                        Dim propIntMaxVal = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_MAX_VALUE)
                                        meError = meIDS.mePropertyGetIntW(propIntMaxVal.ToString(), propIntMax)
                                        If meError = meIDS.ME_ERRNO_SUCCESS Then
                                            nodeText.Append(", Maximum: " & propIntMax.ToString())
                                        End If
                                    End If
                                End If
                            ElseIf propTypeVal = meIDS.ME_PROPERTY_TYPE_DOUBLE Then
                                Dim propDoubleVal As Double
                                nodeText.Append(", Type: Double")
                                meError = meIDS.mePropertyGetDoubleW(propValue.ToString(), propDoubleVal)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then
                                    nodeText.Append(", Value: " & propDoubleVal)
                                End If
                                If propAccType = meIDS.ME_PROPERTY_ACCESS_READ_WRITE Then
                                    Dim propDoubleMin As Double
                                    Dim propDoubleMax As Double
                                    Dim propDoubleMinVal = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_MIN_VALUE)
                                    meError = meIDS.mePropertyGetDoubleW(propDoubleMinVal.ToString(), propDoubleMin)
                                    If meError = meIDS.ME_ERRNO_SUCCESS Then

                                        nodeText.Append(", Minimum: " & propDoubleMin.ToString())
                                        Dim propDoubleMaxVal = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_MAX_VALUE)
                                        meError = meIDS.mePropertyGetDoubleW(propDoubleMaxVal.ToString(), propDoubleMax)
                                        If meError = meIDS.ME_ERRNO_SUCCESS Then
                                            nodeText.Append(", Maximum: " & propDoubleMax)
                                        End If
                                    End If
                                End If
                            ElseIf propTypeVal = meIDS.ME_PROPERTY_TYPE_STRING Then
                                Dim propStringVal = New StringBuilder(Constants.PROPERTY_STRING_LEN)
                                nodeText.Append(", Type: String")
                                meError = meIDS.mePropertyGetStringW(propValue.ToString(), propStringVal, propStringVal.Capacity)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then

                                    nodeText.Append(", Value: """ & propStringVal.ToString() & """")
                                End If
                                Dim propIntLength As Integer
                                Dim propStringLength = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_LENGTH)
                                meError = meIDS.mePropertyGetIntW(propStringLength.ToString(), propIntLength)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then
                                    nodeText.Append(", Length: " & propIntLength.ToString)
                                End If

                                If propAccType = meIDS.ME_PROPERTY_ACCESS_READ_WRITE Then
                                    Dim propIntMaxLength As Integer
                                    Dim propStringMaxLength = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_MAX_LENGTH)
                                    meError = meIDS.mePropertyGetIntW(propStringMaxLength.ToString(), propIntMaxLength)
                                    If meError = meIDS.ME_ERRNO_SUCCESS Then
                                        nodeText.Append(", MaxLength: " & propIntMaxLength.ToString)
                                    End If
                                End If

                            ElseIf propTypeVal = meIDS.ME_PROPERTY_TYPE_DEFINE Then
                                Dim propDefineVal = New StringBuilder(Constants.PROPERTY_STRING_LEN)
                                nodeText.Append(", Type: Define")
                                meError = meIDS.mePropertyGetStringW(propValue.ToString(), propDefineVal, propDefineVal.Capacity)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then
                                    nodeText.Append(", Value: " + propDefineVal.ToString())
                                End If

                                Dim propIntNumSelections As Integer
                                Dim propNumSelections = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_NUMBER_OF_SELECTIONS)
                                meError = meIDS.mePropertyGetIntW(propNumSelections.ToString(), propIntNumSelections)
                                If meError = meIDS.ME_ERRNO_SUCCESS Then
                                    nodeText.Append(String.Format(", Selections: {0} (", propIntNumSelections))
                                    Dim j As Integer
                                    For j = 0 To propIntNumSelections - 1
                                        Dim propSelectionVal = New StringBuilder(Constants.PROPERTY_STRING_LEN)
                                        Dim propSelection = New StringBuilder(containerPath & "\" & propStrVal.ToString() & "\" & meIDS.ME_ATTRIBUTE_NAME_PREFIX_SELECTION & j.ToString())
                                        meError = meIDS.mePropertyGetStringW(propSelection.ToString(), propSelectionVal, propSelectionVal.Capacity)
                                        If meError = meIDS.ME_ERRNO_SUCCESS Then
                                            nodeText.Append(propSelectionVal)
                                            If (j < propIntNumSelections - 1) Then
                                                nodeText.Append(", ")
                                            End If
                                        End If
                                    Next
                                    nodeText.Append(")")
                                End If
                            End If

                            Dim treeNode = New TreeNode(nodeText.ToString())
                            parentNode.Nodes.Add(treeNode)
                        End If
                    End If
                End If
            Next
        End If
    End Sub

    Private Function GetPropertyBoolString(propertyBoolValue As Integer)
        Dim tmpStr As String
        If propertyBoolValue = meIDS.ME_PROPERTY_BOOLEAN_FALSE Then
            tmpStr = "False"
        ElseIf propertyBoolValue = meIDS.ME_PROPERTY_BOOLEAN_TRUE Then
            tmpStr = "True"
        Else
            tmpStr = String.Format("Error boolean value: 0x{0:X}", propertyBoolValue)
        End If
        Return tmpStr
    End Function

    Private Function GetAccessTypeString(propertyAccessType As Integer)
        Dim tmpStr As String
        If propertyAccessType = meIDS.ME_PROPERTY_ACCESS_READ_WRITE Then
            tmpStr = ", Read/Write"
        ElseIf (propertyAccessType = meIDS.ME_PROPERTY_ACCESS_READ_ONLY) Then
            tmpStr = ", Read only"
        Else
            tmpStr = String.Format("Error access type: 0x{0:X}", propertyAccessType)
        End If
        Return tmpStr
    End Function

    Private Sub frmMain_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
            meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS)
        End If
    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub btnAbout_Click(sender As System.Object, e As System.EventArgs) Handles btnAbout.Click
        AboutBox1.ShowDialog()
    End Sub
End Class

Public Class Constants
    Public Shared ReadOnly ME_IDS_NOTOPENED As Integer = -1
    Public Shared ReadOnly PROPERTY_STRING_LEN As Integer = 256
End Class
