﻿'==============================================================================
'Copyright(c), Meilhaus Electronic GmbH
'              http://www.meilhaus.de
'              support@meilhaus.de
'
'Source File : FormMain.vb
'Description : Example program for reading single values from Analog-In (AI) subdevices
'              - Open ME-iDS main driver when program is started
'              - Go through the list of all devices and find AI subdevices
'              - Find AI subdevices and information about channels and ranges
'              - Add AI subdevices to the combobox, so the user can select one
'              - Read data from selected subdevice when user clicks on button "Read Data"
'              - Close main driver when program is closed
'==============================================================================
Imports System.Text


Public Class frmMain
    Private openErrSuccess As Integer = Constants.ME_IDS_NOTOPENED
    Private dataLine As Integer
    Private subdeviceList As New List(Of AiSubdevice)

    Private Sub frmMain_Shown(sender As System.Object, e As System.EventArgs) Handles MyBase.Shown
        OpenMEiDSDriver()
    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub frmMain_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        CloseMEiDSDriver()
    End Sub

    Private Sub OpenMEiDSDriver()
        Try
            textBoxInfo.AppendText("Opening Maindriver..." & Environment.NewLine)
            openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS)

            If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
                DisplayVersionLibrary()
                DisplayVersionMainDriver()
                textBoxInfo.AppendText(Environment.NewLine)

                Dim numDevices As Integer
                meIDS.meQueryNumberDevices(numDevices)
                textBoxInfo.AppendText("Number of devices found: " & numDevices.ToString() & Environment.NewLine)

                GetAiSubdevices(numDevices)
            Else
                DisplayErrorMessage("meOpen - Error: ", openErrSuccess)
            End If
        Catch ex As DllNotFoundException
            textBoxInfo.AppendText("The ME-iDS is not properly installed." & Environment.NewLine)
            textBoxInfo.AppendText(ex.Message & Environment.NewLine)
        End Try
    End Sub

    Private Sub GetAiSubdevices(numDevices As Integer)
        Dim idxDevice As Integer
        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS AndAlso numDevices > 0 Then
            textBoxInfo.AppendText("Retrieving sub-devices of type ME_TYPE_AI (Analog input)..." & Environment.NewLine)
            For idxDevice = 0 To numDevices - 1
                Dim meError As Integer
                Dim plugged As Integer

                meError = GetDevicePlugged(idxDevice, plugged)
                If meError = meIDS.ME_ERRNO_SUCCESS AndAlso plugged = meIDS.ME_PLUGGED_IN Then
                    GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_AI)
                Else
                    textBoxInfo.AppendText("Device " & idxDevice.ToString() & " not plugged in" & Environment.NewLine)
                End If
            Next
        End If
        DisplaySubdevices()
    End Sub

    Private Sub GetSubdevicesByType(idxDevice As Integer, subdeviceType As Integer)
        Dim meError As Integer
        Dim numSubdevices As Integer
        Dim idxMatchedSubdevice As Integer

        meError = meIDS.meQueryNumberSubdevices(idxDevice, numSubdevices)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            Dim deviceName = New StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT)
            meError = meIDS.meQueryNameDevice(idxDevice, deviceName, meIDS.ME_DEVICE_NAME_MAX_COUNT)
            Dim idxSubdevice As Integer = 0
            While idxSubdevice < numSubdevices
                meError = meIDS.meQuerySubdeviceByType(idxDevice, _
                                                 idxSubdevice, _
                                                 subdeviceType, _
                                                 meIDS.ME_SUBTYPE_ANY, _
                                                 idxMatchedSubdevice)

                If meError = meIDS.ME_ERRNO_SUCCESS Then
                    AddAiSubdevice(deviceName.ToString(), idxDevice, idxMatchedSubdevice, subdeviceType)
                    idxSubdevice = idxMatchedSubdevice + 1
                Else
                    Exit While
                End If
            End While
        End If
    End Sub

    Private Function AddAiSubdevice(deviceName As String, idxDevice As Integer, idxSubdevice As Integer, subdeviceType As Integer) As Integer
        Dim meError As Integer
        Dim numOfChannels As Integer
        meIDS.meQueryNumberChannels(idxDevice, idxSubdevice, numOfChannels)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            Dim aiSubdev As AiSubdevice = New AiSubdevice(deviceName.ToString, idxDevice, idxSubdevice, numOfChannels)
            subdeviceList.Add(aiSubdev)
            AddAnalogRanges(aiSubdev)
            coBoxSubDevs.Items.Add(deviceName.ToString _
                                        & ",  Device " & idxDevice.ToString() _
                                        & ",  Subdevice " & idxSubdevice.ToString() _
                                        & ",  Type: " & GetSubDevTypeName(subdeviceType) _
                                        & ",  Channels: " & numOfChannels.ToString() _
                                        & ",  Ranges: " & aiSubdev.GetRangeCount().ToString())
        End If
        Return meError
    End Function

    Private Sub AddAnalogRanges(aiSubdev As AiSubdevice)
        Dim numOfRanges As Integer
        Dim idxRange As Integer
        meIDS.meQueryNumberRanges(aiSubdev.deviceIndex, _
                                  aiSubdev.subdevIndex, _
                                  meIDS.ME_UNIT_ANY, _
                                  numOfRanges)
        idxRange = 0
        While (idxRange < numOfRanges)
            AddAnalogRange(aiSubdev, idxRange)
            idxRange = idxRange + 1
        End While

    End Sub

    Private Sub AddAnalogRange(aiSubdev As AiSubdevice, idxRange As Integer)
        Dim unit As Integer
        Dim physMin As Double
        Dim physMax As Double
        Dim digitalMax As Integer
        Dim meError As Integer = meIDS.meQueryRangeInfo(aiSubdev.deviceIndex, _
                                   aiSubdev.subdevIndex, _
                                   idxRange, _
                                   unit, _
                                   physMin, _
                                   physMax, _
                                   digitalMax)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            aiSubdev.AddRange(unit, digitalMax, physMin, physMax)
        End If
    End Sub

    Private Sub btnResetSubdev_Click(sender As System.Object, e As System.EventArgs) Handles btnResetSubdev.Click
        Dim aiSubdev As AiSubdevice
        Dim meError As Integer
        aiSubdev = subdeviceList(coBoxSubDevs.SelectedIndex)
        meError = meIDS.meIOResetSubdevice(aiSubdev.deviceIndex, _
                                     aiSubdev.subdevIndex, _
                                     meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            textBoxInfo.AppendText("Reset Device " & aiSubdev.deviceIndex.ToString() & ", Subdevice " & aiSubdev.subdevIndex.ToString() & Environment.NewLine)
        Else
            DisplayErrorMessage("Error resetting Subdevice ", meError)
        End If
    End Sub

    Private Sub btnSingleRead_Click(sender As System.Object, e As System.EventArgs) Handles btnSingleRead.Click
        If (coBoxSubDevs.SelectedIndex >= 0) Then
            Dim meError As Integer
            Dim aiSubdev As AiSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex)
            Dim aiRange As AnalogRange = aiSubdev.GetAnalogRange(coBoxRange.SelectedIndex)
            Dim idxChannel As Integer = coBoxChannel.SelectedIndex
            Dim idxRange As Integer = coBoxRange.SelectedIndex

            meError = ConfigureSubdeviceForInput(aiSubdev, idxChannel, idxRange)
            If meError = meIDS.ME_ERRNO_SUCCESS Then
                Dim valDigital As Integer
                ReadSingleValue(aiSubdev, idxChannel, valDigital)
                If meError = meIDS.ME_ERRNO_SUCCESS Then
                    Dim valAnalog As Double
                    meError = ConvertDigitalToPhysical(aiSubdev, idxRange, valDigital, valAnalog)
                    If meError = meIDS.ME_ERRNO_SUCCESS Then
                        DisplayValue(aiSubdev, idxChannel, coBoxRange.SelectedIndex, valDigital, valAnalog)
                    Else
                        DisplayErrorMessage("meUtilityDigitalToPhysical - Error: ", meError)
                    End If
                Else
                    DisplayErrorMessage("meIOSingle - Error: ", meError)
                End If
            Else
                DisplayErrorMessage("meIOSingleConfig - Error: ", meError)
            End If
        End If
    End Sub

    Private Function ConfigureSubdeviceForInput(aiSubdev As AiSubdevice, idxChannel As Integer, idxRange As Integer) As Integer
        Dim meError As Integer
        meError = meIDS.meIOSingleConfig(aiSubdev.deviceIndex, _
                                  aiSubdev.subdevIndex, _
                                  idxChannel, _
                                  idxRange, _
                                  meIDS.ME_REF_AI_GROUND, _
                                  meIDS.ME_TRIG_CHAN_DEFAULT, _
                                  meIDS.ME_TRIG_TYPE_SW, _
                                  meIDS.ME_VALUE_NOT_USED, _
                                  meIDS.ME_IO_SINGLE_CONFIG_NO_FLAGS)
        Return meError
    End Function

    Private Function ReadSingleValue(aiSubdev As AiSubdevice, ByVal idxChannel As Integer, ByRef valDigital As Integer) As Integer
        Dim io_single(1) As meIDS.meIOSingle_t
        Dim meError As Integer

        io_single(0).iDevice = aiSubdev.deviceIndex
        io_single(0).iSubdevice = aiSubdev.subdevIndex
        io_single(0).iChannel = idxChannel
        io_single(0).iDir = meIDS.ME_DIR_INPUT
        io_single(0).iValue = 0
        io_single(0).iTimeOut = 0
        io_single(0).iFlags = meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS
        io_single(0).iErrno = 0

        meError = meIDS.meIOSingle(io_single(0), _
                                       1, _
                                       meIDS.ME_IO_SINGLE_NO_FLAGS)
        valDigital = io_single(0).iValue
        Return meError
    End Function

    Private Shared Function ConvertDigitalToPhysical(aiSubdev As AiSubdevice, idxRange As Integer, ByVal valDigital As Integer, ByRef valAnalog As Double) As Integer
        Dim meError As Integer
        Dim physMin As Double = aiSubdev.GetAnalogRange(idxRange).physicalMin
        Dim physMax As Double = aiSubdev.GetAnalogRange(idxRange).physicalMax
        Dim digitalMax As Integer = aiSubdev.GetAnalogRange(idxRange).digitalMax
        meError = meIDS.meUtilityDigitalToPhysical(physMin, _
                                            physMax, _
                                            digitalMax, _
                                            valDigital, _
                                            meIDS.ME_VALUE_NOT_USED, _
                                            0.0, _
                                            valAnalog)
        Return meError
    End Function

    Private Sub DisplayValue(aiSubdev As AiSubdevice, idxChannel As Integer, idxRange As Integer, valDigital As Integer, valAnalog As Double)
        dataLine = dataLine + 1
        textBoxInfo.AppendText(String.Format("{0:00}", dataLine) _
                            & " Read from " & aiSubdev.deviceName _
                            & ",  Device: " & aiSubdev.deviceIndex.ToString() _
                            & ",  Subdevice: " & aiSubdev.subdevIndex.ToString() _
                            & ",  Channel: " & idxChannel.ToString() _
                            & ",  Digital value: " & valDigital.ToString() _
                            & ",  Physical value: " & String.Format("{0:0.000}", valAnalog) _
                            & " " + aiSubdev.GetAnalogRange(idxRange).displayUnit & Environment.NewLine)
    End Sub

    Private Sub coBoxSubDevs_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxSubDevs.SelectedIndexChanged
        FillSubdeviceProperties(coBoxSubDevs.SelectedIndex)
    End Sub

    Private Sub FillSubdeviceProperties(idxSelectedSubdev As Integer)
        coBoxChannel.Items.Clear()
        coBoxRange.Items.Clear()
        If idxSelectedSubdev >= 0 Then
            Dim aiSubdev As AiSubdevice = subdeviceList(idxSelectedSubdev)
            FillSubdeviceChannels(aiSubdev)
            FillSubdeviceRanges(aiSubdev)
        End If
    End Sub

    Private Sub FillSubdeviceChannels(ByVal aiSubdev As AiSubdevice)
        Dim i As Integer
        For i = 0 To aiSubdev.numOfChannels - 1
            coBoxChannel.Items.Add("Channel " & i.ToString())
        Next
        coBoxChannel.SelectedIndex = 0
    End Sub

    Private Sub FillSubdeviceRanges(ByVal aiSubdev As AiSubdevice)
        Dim i As Integer
        Dim aiRange As AnalogRange
        For i = 0 To aiSubdev.GetRangeCount() - 1
            aiRange = aiSubdev.GetAnalogRange(i)
            coBoxRange.Items.Add("Range " & i.ToString() & ": " & aiRange.displayRange)
        Next
        coBoxRange.SelectedIndex = 0
    End Sub

    Private Sub DisplaySubdevices()
        If coBoxSubDevs.Items.Count > 0 Then
            textBoxInfo.AppendText("Number of AI sub-devices found: " & coBoxSubDevs.Items.Count.ToString() & Environment.NewLine)
            Dim i As Integer
            For i = 0 To coBoxSubDevs.Items.Count - 1
                textBoxInfo.AppendText(coBoxSubDevs.Items(i) & Environment.NewLine)
            Next
            textBoxInfo.AppendText(Environment.NewLine)
            coBoxSubDevs.SelectedIndex = 0
            btnSingleRead.Enabled = True
            btnResetSubdev.Enabled = True
        Else
            textBoxInfo.AppendText("No AI sub-devices found in system." & Environment.NewLine)
            CloseMEiDSDriver()
        End If
    End Sub

    Private Function GetSubDevTypeName(subdeviceType As Integer)
        Dim tmpType As String
        Select Case subdeviceType
            Case meIDS.ME_TYPE_INVALID
                tmpType = "ME_TYPE_INVALID"
            Case meIDS.ME_TYPE_AI
                tmpType = "ME_TYPE_AI"
            Case meIDS.ME_TYPE_AO
                tmpType = "ME_TYPE_AO"
            Case meIDS.ME_TYPE_DIO
                tmpType = "ME_TYPE_DIO"
            Case meIDS.ME_TYPE_DO
                tmpType = "ME_TYPE_DO"
            Case meIDS.ME_TYPE_DI
                tmpType = "ME_TYPE_DI"
            Case meIDS.ME_TYPE_CTR
                tmpType = "ME_TYPE_CTR"
            Case meIDS.ME_TYPE_EXT_IRQ
                tmpType = "ME_TYPE_EXT_IRQ"
            Case meIDS.ME_TYPE_FIO
                tmpType = "ME_TYPE_FIO"
            Case meIDS.ME_TYPE_FO
                tmpType = "ME_TYPE_FO"
            Case meIDS.ME_TYPE_FI
                tmpType = "ME_TYPE_FI"
            Case meIDS.ME_TYPE_FPGA
                tmpType = "ME_TYPE_FPGA"
            Case Else
                tmpType = String.Format("&H{0:X}", subdeviceType)
        End Select
        Return tmpType
    End Function

    Private Sub CloseMEiDSDriver()
        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
            meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS)
            openErrSuccess = Constants.ME_IDS_NOTOPENED
            btnSingleRead.Enabled = False
            btnResetSubdev.Enabled = False
            subdeviceList.Clear()
            coBoxSubDevs.Items.Clear()
            coBoxChannel.Items.Clear()
            coBoxRange.Items.Clear()
        End If
    End Sub

    Private Function GetDevicePlugged(idxDevice As Integer, ByRef plugged As Integer) As Integer

        Dim meError As Integer
        Dim vendorId As Integer
        Dim deviceId As Integer
        Dim serialNo As Integer
        Dim busType As Integer
        Dim busNo As Integer
        Dim devNo As Integer
        Dim funcNo As Integer

        meError = meIDS.meQueryInfoDevice(idxDevice, _
                                    vendorId, _
                                    deviceId, _
                                    serialNo, _
                                    busType, _
                                    busNo, _
                                    devNo, _
                                    funcNo, _
                                    plugged)
        Return meError
    End Function

    Private Sub DisplayVersionLibrary()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionLibrary(tmpVersion)
        textBoxInfo.AppendText("Library version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Sub DisplayVersionMainDriver()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionMainDriver(tmpVersion)
        textBoxInfo.AppendText("Maindriver version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Sub DisplayErrorMessage(message As String, meError As Integer)
        Dim errMessage = New StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText(message & meError.ToString() & Environment.NewLine)
        meIDS.meErrorGetMessage(meError, errMessage, meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText("ErrorMessage: " & errMessage.ToString & Environment.NewLine)
    End Sub

    Private Sub btnAbout_Click(sender As System.Object, e As System.EventArgs) Handles btnAbout.Click
        AboutBox1.ShowDialog()
    End Sub

    Private Sub btnClearHwInfo_Click(sender As System.Object, e As System.EventArgs) Handles btnClearHwInfo.Click
        textBoxInfo.Clear()
        dataLine = 0
    End Sub

    Private Function MeVersionToString(version As Integer) As String
        Return String.Format("{0}.{1}.{2}.{3}", version >> 24, &HFF And (version >> 16), &HFF And (version >> 8), &HFF And version)
    End Function
End Class

Public Class AiSubdevice
    Public deviceName As String
    Public deviceIndex As Integer
    Public subdevIndex As Integer
    Public numOfChannels As Integer
    Private analogRanges As New List(Of AnalogRange)

    Sub New(DevName As String, DeviceIdx As Integer, SubdevIdx As Integer, NumChannels As Integer)
        deviceName = DevName
        deviceIndex = DeviceIdx
        subdevIndex = SubdevIdx
        numOfChannels = NumChannels
    End Sub

    Public Sub AddRange(RangeUnit As Integer, DigitalMax As Integer, PhysicalMin As Double, PhysicalMax As Double)
        Dim aiRange As AnalogRange = New AnalogRange(RangeUnit, DigitalMax, PhysicalMin, PhysicalMax)
        analogRanges.Add(aiRange)
    End Sub

    Public Function GetAnalogRange(index As Integer) As AnalogRange
        Return analogRanges(index)
    End Function

    Public Function GetRangeCount() As Integer
        Return analogRanges.Count
    End Function
End Class

Public Class AnalogRange
    Public rangeUnit As Integer
    Public digitalMax As Integer
    Public physicalMin As Double
    Public physicalMax As Double
    Public displayRange As String
    Public displayUnit As String

    Sub New(Unit As Integer, DigitMax As Integer, PhysMin As Double, PhysMax As Double)
        rangeUnit = Unit
        digitalMax = DigitMax
        physicalMin = PhysMin
        physicalMax = PhysMax
        SetRangeUnitTexts()
    End Sub

    Private Sub SetRangeUnitTexts()
        Dim sbRange As New StringBuilder
        Dim tmp As String = ""
        Dim tmpValMin As Double = Math.Abs(physicalMin)
        Dim tmpValMax As Double = Math.Abs(physicalMax)

        If (rangeUnit = meIDS.ME_UNIT_VOLT) Then
            displayUnit = "Volt"
            tmp = "V"
        ElseIf (rangeUnit = meIDS.ME_UNIT_AMPERE) Then
            displayUnit = "Ampere"
            tmp = "A"
        End If

        If (tmpValMin = 0) Then
            sbRange.Append("0")
        ElseIf (tmpValMin < 0.001) Then
            sbRange.Append(String.Format("{0:0.00}", physicalMin * 1000000))
            sbRange.Append("µ")
        ElseIf (tmpValMin < 1) Then
            sbRange.Append(String.Format("{0:0.00}", physicalMin * 1000))
            sbRange.Append("m")
        Else
            sbRange.Append(String.Format("{0:0.00}", Math.Round(physicalMin, 2)))
        End If
        sbRange.Append(tmp)


        If (tmpValMax = 0) Then
            sbRange.Append("...0")
        ElseIf (tmpValMax < 0.001) Then
            sbRange.Append(String.Format("...{0:0.00}", physicalMax * 1000000))
            sbRange.Append("µ")
        ElseIf (tmpValMax < 1) Then
            sbRange.Append(String.Format("...{0:0.00}", physicalMax * 1000))
            sbRange.Append("m")
        Else
            sbRange.Append(String.Format("...{0:0.00}", Math.Round(physicalMax, 2)))
        End If
        sbRange.Append(tmp)

        displayRange = sbRange.ToString()
    End Sub
End Class

Class Constants
    Public Shared ReadOnly ME_IDS_NOTOPENED As Integer = -1
End Class





