﻿'==============================================================================
'Copyright(c), Meilhaus Electronic GmbH
'              http://www.meilhaus.de
'              support@meilhaus.de
'
'Source File : FormMain.vb
'Description : Example program for reading single values from DI and DIO devices
'              - Open main driver (ME-iDS)
'              - Go through the list of all devices and find DI and DIO subdevices
'              - Add DI and DIO devices to the combobox, so the user can select one
'              - Read data from selected subdevice when user clicks on button "Read Data"
'              - Data are always read from channel 0 in this simple example
'              - Close main driver when program is closed if not already done
'==============================================================================
Imports System.Text

Public Class frmMain
    Private openErrSuccess As Integer = Constants.ME_IDS_NOTOPENED
    Private dataLine As Integer
    Private subdeviceList As New List(Of DiSubdevice)


    Private Sub frmMain_Shown(sender As System.Object, e As System.EventArgs) Handles MyBase.Shown
        OpenMEiDSDriver()
    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub frmMain_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        CloseMEiDSDriver()
    End Sub

    Private Sub OpenMEiDSDriver()
        Try

            textBoxInfo.AppendText("Opening Maindriver..." & Environment.NewLine)
            openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS)

            If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
                DisplayVersionLibrary()
                DisplayVersionMainDriver()

                Dim numDevices As Integer
                meIDS.meQueryNumberDevices(numDevices)
                textBoxInfo.AppendText("Number of devices found: " & numDevices.ToString() & Environment.NewLine)
                GetDiDioSubdevices(numDevices)
            Else
                DisplayErrorMessage("meOpen - Error: ", openErrSuccess)
            End If
        Catch ex As DllNotFoundException
            textBoxInfo.AppendText("The ME-iDS is not properly installed." & Environment.NewLine)
            textBoxInfo.AppendText(ex.Message & Environment.NewLine)
        End Try
    End Sub

    Private Sub GetDiDioSubdevices(numDevices As Integer)
        Dim idxDevice As Integer
        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS AndAlso numDevices > 0 Then
            textBoxInfo.AppendText("Retrieving sub-devices of type ME_TYPE_DI (Digital In) and ME_TYPE_DIO (Digital In/Out)..." & Environment.NewLine)

            For idxDevice = 0 To numDevices - 1
                Dim meError As Integer
                Dim plugged As Integer
                meError = GetDevicePlugged(idxDevice, plugged)
                If meError = meIDS.ME_ERRNO_SUCCESS AndAlso plugged = meIDS.ME_PLUGGED_IN Then
                    GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_DI)
                    GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_DIO)
                Else
                    textBoxInfo.AppendText("Device " & idxDevice.ToString() & " not plugged in" & Environment.NewLine)
                End If
            Next
        End If
        DisplaySubdevices()
    End Sub

    Private Sub GetSubdevicesByType(idxDevice As Integer, subdeviceType As Integer)
        Dim meError As Integer
        Dim numSubdevices As Integer
        Dim idxMatchedSubdevice As Integer

        meError = meIDS.meQueryNumberSubdevices(idxDevice, numSubdevices)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            Dim idxSubdevice As Integer = 0
            Dim deviceName = New StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT)
            meIDS.meQueryNameDevice(idxDevice, deviceName, meIDS.ME_DEVICE_NAME_MAX_COUNT)
            While idxSubdevice < numSubdevices

                meError = meIDS.meQuerySubdeviceByType(idxDevice, _
                                                 idxSubdevice, _
                                                 subdeviceType, _
                                                 meIDS.ME_SUBTYPE_ANY, _
                                                 idxMatchedSubdevice)

                If meError = meIDS.ME_ERRNO_SUCCESS Then
                    AddDiSubdevice(deviceName.ToString(), idxDevice, idxMatchedSubdevice, subdeviceType)
                    idxSubdevice = idxMatchedSubdevice + 1
                Else
                    Exit While
                End If
            End While
        End If
    End Sub

    Private Sub AddDiSubdevice(ByVal deviceName As String, ByVal idxDevice As Integer, ByVal idxSubdevice As Integer, ByVal subdeviceType As Integer)
        Dim numOfChannels As Integer

        meIDS.meQueryNumberChannels(idxDevice, idxSubdevice, numOfChannels)
        subdeviceList.Add(New DiSubdevice(deviceName, idxDevice, idxSubdevice, numOfChannels))

        coBoxSubDevs.Items.Add(deviceName _
                                    & ",  Device " & idxDevice.ToString() _
                                    & ",  Subdevice " & idxSubdevice.ToString() _
                                    & ",  Type: " & GetSubDevTypeName(subdeviceType) _
                                    & ",  Channels: " & numOfChannels.ToString())
    End Sub

    Private Sub btnSingleRead_Click(sender As System.Object, e As System.EventArgs) Handles btnSingleRead.Click
        If coBoxSubDevs.SelectedIndex >= 0 Then
            Dim meError As Integer
            Dim diSubdevice As DiSubdevice
            diSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex)

            meError = ConfigureSubdeviceForInput(diSubdevice)
            If meError = meIDS.ME_ERRNO_SUCCESS Then
                Dim value As Integer
                meError = ReadSingleValue(diSubdevice, value)
                If meError = meIDS.ME_ERRNO_SUCCESS Then
                    DisplayValue(diSubdevice, value)
                Else
                    DisplayErrorMessage("meIOSingle - Error: ", meError)
                End If
            Else
                DisplayErrorMessage("meIOSingleConfig - Error: ", meError)
            End If
        End If
    End Sub

    Private Shared Function ConfigureSubdeviceForInput(ByVal diSubdevice As DiSubdevice) As Integer
        Dim meError As Integer
        meError = meIDS.meIOSingleConfig(diSubdevice.deviceIndex, _
                                        diSubdevice.subdevIndex, _
                                        0, _
                                        meIDS.ME_SINGLE_CONFIG_DIO_INPUT, _
                                        meIDS.ME_REF_NONE, _
                                        meIDS.ME_VALUE_NOT_USED, _
                                        meIDS.ME_TRIG_TYPE_NONE, _
                                        meIDS.ME_VALUE_NOT_USED, _
                                        diSubdevice.singleConfigFlag)
        Return meError
    End Function

    Private Function ReadSingleValue(ByVal diSubdevice As DiSubdevice, ByRef value As Integer)
        Dim io_single(1) As meIDS.meIOSingle_t

        io_single(0).iDevice = diSubdevice.deviceIndex
        io_single(0).iSubdevice = diSubdevice.subdevIndex
        io_single(0).iChannel = 0
        io_single(0).iDir = meIDS.ME_DIR_INPUT
        io_single(0).iValue = 0
        io_single(0).iTimeOut = 0
        io_single(0).iFlags = diSubdevice.singleTypeFlag
        io_single(0).iErrno = 0

        Dim meError As Integer = meIDS.meIOSingle(io_single(0), _
                                                1, _
                                                meIDS.ME_IO_SINGLE_NO_FLAGS)

        value = io_single(0).iValue
        Return meError
    End Function

    Private Sub DisplayValue(ByVal diSubdevice As DiSubdevice, value As Integer)
        dataLine = dataLine + 1
        textBoxInfo.AppendText(String.Format("{0:00}", dataLine) _
                  & " Read from " & diSubdevice.deviceName _
                  & ",  Device: " & diSubdevice.deviceIndex.ToString() _
                  & ",  Subdevice: " & diSubdevice.subdevIndex.ToString() _
                  & String.Format(",  Value hex: &H{0:X}", value) _
                  & ",  bin: " & IntToBin(value, diSubdevice.numOfChannels) _
                  & ",  dec: " & value.ToString() & Environment.NewLine)
    End Sub

    Private Sub btnResetSubdev_Click(sender As System.Object, e As System.EventArgs) Handles btnResetSubdev.Click
        Dim diSubdev As DiSubdevice
        Dim meError As Integer

        diSubdev = subdeviceList(coBoxSubDevs.SelectedIndex)
        meError = meIDS.meIOResetSubdevice(diSubdev.deviceIndex, _
                                     diSubdev.subdevIndex, _
                                     meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            textBoxInfo.AppendText("Reset Device " & diSubdev.deviceIndex.ToString() & ", Subdevice " & diSubdev.subdevIndex.ToString() & Environment.NewLine)
        Else
            DisplayErrorMessage("Error resetting Subdevice ", meError)
        End If
    End Sub

    Private Function GetSubDevTypeName(subdeviceType As Integer)
        Dim tmpType As String
        Select Case subdeviceType
            Case meIDS.ME_TYPE_INVALID
                tmpType = "ME_TYPE_INVALID"
            Case meIDS.ME_TYPE_AI
                tmpType = "ME_TYPE_AI"
            Case meIDS.ME_TYPE_AO
                tmpType = "ME_TYPE_AO"
            Case meIDS.ME_TYPE_DIO
                tmpType = "ME_TYPE_DIO"
            Case meIDS.ME_TYPE_DO
                tmpType = "ME_TYPE_DO"
            Case meIDS.ME_TYPE_DI
                tmpType = "ME_TYPE_DI"
            Case meIDS.ME_TYPE_CTR
                tmpType = "ME_TYPE_CTR"
            Case meIDS.ME_TYPE_EXT_IRQ
                tmpType = "ME_TYPE_EXT_IRQ"
            Case meIDS.ME_TYPE_FIO
                tmpType = "ME_TYPE_FIO"
            Case meIDS.ME_TYPE_FO
                tmpType = "ME_TYPE_FO"
            Case meIDS.ME_TYPE_FI
                tmpType = "ME_TYPE_FI"
            Case meIDS.ME_TYPE_FPGA
                tmpType = "ME_TYPE_FPGA"
            Case Else
                tmpType = String.Format("&H{0:X}", subdeviceType)
        End Select
        Return tmpType
    End Function

    Private Function IntToBin(val As Integer, dispDigits As Integer) As String
        Dim b As Integer
        Dim found As Boolean = False
        Dim tmpStr As String = ""
        For b = 0 To Constants.DIGITNUMS.Length - 1
            If dispDigits = Constants.DIGITNUMS(b) Then
                found = True
                Exit For
            End If
        Next
        If found Then
            If dispDigits = 1 Then
                tmpStr = (val And 1).ToString()
            Else
                For i As Integer = (dispDigits >> 2) - 1 To 0 Step -1
                    b = (val >> (i << 2)) And &HF
                    tmpStr = tmpStr & Constants.BIN_STR(b)
                Next
            End If
        End If
        Return tmpStr
    End Function

    Private Sub DisplayErrorMessage(message As String, meError As Integer)
        Dim errMessage = New StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText(message & meError.ToString() & Environment.NewLine)
        meIDS.meErrorGetMessage(meError, errMessage, meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText("ErrorMessage: " & errMessage.ToString & Environment.NewLine)
    End Sub

    Private Sub CloseMEiDSDriver()
        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
            meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS)
            openErrSuccess = Constants.ME_IDS_NOTOPENED
            btnSingleRead.Enabled = False
            btnResetSubdev.Enabled = False
            subdeviceList.Clear()
            coBoxSubDevs.Items.Clear()
        End If
    End Sub

    Private Sub btnClearHwInfo_Click(sender As System.Object, e As System.EventArgs) Handles btnClearHwInfo.Click
        textBoxInfo.Clear()
        dataLine = 0
    End Sub

    Private Function GetDevicePlugged(idxDevice As Integer, ByRef plugged As Integer) As Integer

        Dim meError As Integer
        Dim vendorId As Integer
        Dim deviceId As Integer
        Dim serialNo As Integer
        Dim busType As Integer
        Dim busNo As Integer
        Dim devNo As Integer
        Dim funcNo As Integer

        meError = meIDS.meQueryInfoDevice(idxDevice, _
                                    vendorId, _
                                    deviceId, _
                                    serialNo, _
                                    busType, _
                                    busNo, _
                                    devNo, _
                                    funcNo, _
                                    plugged)
        Return meError
    End Function

    Private Sub DisplaySubdevices()
        If coBoxSubDevs.Items.Count > 0 Then
            textBoxInfo.AppendText("Number of DI or DIO sub-devices found: " & coBoxSubDevs.Items.Count.ToString() & Environment.NewLine)
            Dim i As Integer
            For i = 0 To coBoxSubDevs.Items.Count - 1
                textBoxInfo.AppendText(coBoxSubDevs.Items(i) & Environment.NewLine)
            Next
            textBoxInfo.AppendText(Environment.NewLine)
            coBoxSubDevs.SelectedIndex = 0
            btnSingleRead.Enabled = True
            btnResetSubdev.Enabled = True
        Else
            textBoxInfo.AppendText("No DI or DIO sub-devices found." & Environment.NewLine)
            CloseMEiDSDriver()
        End If
    End Sub

    Private Sub DisplayVersionLibrary()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionLibrary(tmpVersion)
        textBoxInfo.AppendText("Library version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Sub DisplayVersionMainDriver()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionMainDriver(tmpVersion)
        textBoxInfo.AppendText("Maindriver version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
        textBoxInfo.AppendText(Environment.NewLine)
    End Sub

    Private Function MeVersionToString(version As Integer) As String
        Return String.Format("{0}.{1}.{2}.{3}", version >> 24, &HFF And (version >> 16), &HFF And (version >> 8), &HFF And version)
    End Function

    Private Sub btnAbout_Click(sender As System.Object, e As System.EventArgs) Handles btnAbout.Click
        AboutBox1.ShowDialog()
    End Sub
End Class

Public Class DiSubdevice
    Public deviceName As String
    Public deviceIndex As Integer
    Public subdevIndex As Integer
    Public numOfChannels As Integer
    Public singleConfigFlag As Integer
    Public singleTypeFlag As Integer

    Sub New(DevName As String, DeviceIdx As Integer, SubdevIdx As Integer, NumChannels As Integer)
        deviceName = DevName
        deviceIndex = DeviceIdx
        subdevIndex = SubdevIdx
        numOfChannels = NumChannels
        SetSingleFlags(NumChannels)
    End Sub

    Private Sub SetSingleFlags(NumberOfChannels As Integer)
        Select Case NumberOfChannels
            Case 1
                singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_BIT
                singleTypeFlag = meIDS.ME_IO_SINGLE_TYPE_DIO_BIT
            Case 8
                singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_BYTE
                singleTypeFlag = meIDS.ME_IO_SINGLE_TYPE_DIO_BYTE
            Case 16
                singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_WORD
                singleTypeFlag = meIDS.ME_IO_SINGLE_TYPE_DIO_WORD
            Case 24
                singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_DWORD
                singleTypeFlag = meIDS.ME_IO_SINGLE_TYPE_DIO_DWORD
            Case 32
                singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_DWORD
                singleTypeFlag = meIDS.ME_IO_SINGLE_TYPE_DIO_DWORD
            Case Else
                singleConfigFlag = meIDS.ME_VALUE_INVALID
                singleTypeFlag = meIDS.ME_VALUE_INVALID
        End Select
    End Sub
End Class

Class Constants
    Public Shared ReadOnly ME_IDS_NOTOPENED As Integer = -1
    Public Shared ReadOnly BIN_STR As String() = {"0000", "0001", "0010", "0011",
                                               "0100", "0101", "0110", "0111",
                                               "1000", "1001", "1010", "1011",
                                               "1100", "1101", "1110", "1111"}
    Public Shared ReadOnly DIGITNUMS As Integer() = {1, 8, 16, 24, 32}
End Class
