﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using meIDS_CS;
using meIDSTools;

namespace TestConsole_meIOStreamRead_CS
{
    class TestConsole_meIOStreamRead
    {
        static void Main(string[] args)
        {
            int i_number_of_devices = TestConsoleGeneric.TestConsoleMEIDSProlog();

            if (i_number_of_devices <= 0)
            {
                return;
            }

	        // ---------------------------------------------------------------------
        	
        	int i_stream_read_device	= -1;
	        int i_stream_read_subdevice = -1;

	        // Search for an AI-Streaming subdevice in the system

	        int i_me_error = 0;

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;
        		
		        i_me_error = meIDS.meQueryInfoDevice(	index_device,				// Device index
										                out i_vendor_id,			// Vendor ID returned here				
										                out i_device_id,			// Device ID returned here
										                out i_serial_no,			// Serial number returned here
										                out i_bus_type,				// Bus type returned here
										                out i_bus_no,				// Bus number returned here
										                out i_dev_no,				// Bus device number returned here
										                out i_func_no,				// Bus function number returned here
										                out i_plugged			);	// Plugged status returned here


		        if(i_plugged != meIDS.ME_PLUGGED_IN)
		        {
			        continue;
		        }

		        i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					// Device
											                0,								// Begin search at this sub-device 
											                meIDS.ME_TYPE_AI,				// Type of sub-device to search for
											                meIDS.ME_SUBTYPE_STREAMING,		// Sub-type of sub-device to search for
											                out i_stream_read_subdevice	);	// sub-device index returned here

		        if(i_me_error == meIDS.ME_ERRNO_SUCCESS)
		        {
			        i_stream_read_device = index_device;

			        break;
		        }
	        }

	        if(i_stream_read_device == -1)
	        {
                System.Console.Write("****   No AI-Streaming sub-device found in system    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
        		
		        return;
	        }

	        System.Console.Write("Device: {0}, Subdevice: {1} is an AI-Streaming device\n\n", i_stream_read_device, i_stream_read_subdevice);
        	
	        int i_acq_ticks_low = 0;
	        int i_acq_ticks_high = 0;

	        double d_time = 0.0;

	        i_me_error = meIDS.meIOStreamTimeToTicks(	i_stream_read_device,						// Device index
										                i_stream_read_subdevice,					// Subdevice indexe, 
										                meIDS.ME_TIMER_ACQ_START,					// Timer used 
										                ref d_time,							        // Required time seconds, Achieved time returned here	 
										                out i_acq_ticks_low,					    // Lower 32 bits of the corresponding tick value returned here
										                out i_acq_ticks_high,					    // Upper 32 bits of the corresponding tick value returned here
										                meIDS.ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS);   // No flags

	        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    meIOStreamTimeToTicks - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
				
		        return;
	        }

	        double d_required_freq = 10.0;
	        double d_achieved_freq = d_required_freq;

	        int i_scan_ticks_low = 0;
	        int i_scan_ticks_high = 0;

	        i_me_error = meIDS.meIOStreamFrequencyToTicks(  i_stream_read_device,				        // Device index
											                i_stream_read_subdevice,			        // Subdevice indexe, 
											                meIDS.ME_TIMER_SCAN_START,				    // Timer used 
											                ref d_achieved_freq,					    // Required frequency Hz, Achieved frequency returned here	 
											                out i_scan_ticks_low,					    // Lower 32 bits of the corresponding tick value returned here
											                out i_scan_ticks_high,					    // Upper 32 bits of the corresponding tick value returned here
											                meIDS.ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS);   // No flags

	        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    meIOStreamFrequencyToTicks - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
				
		        return;
	        }

	        System.Console.Write("meIOStreamFrequencyToTicks - Required Freq: {0:F2} Achieved Freq: {1:F2} Ticks low: {2} Ticks high: {3}\n\n", d_required_freq, d_achieved_freq, i_scan_ticks_low, i_scan_ticks_high);

	        int i_conv_ticks_low = 0;
	        int i_conv_ticks_high = 0;

	        d_time = 0.0;

	        i_me_error = meIDS.meIOStreamTimeToTicks(	i_stream_read_device,				        // Device index
										                i_stream_read_subdevice,			        // Subdevice indexe, 
										                meIDS.ME_TIMER_CONV_START,				    // Timer used 
										                ref d_time,							        // Required time seconds, Achieved time returned here	 
										                out i_conv_ticks_low,					    // Lower 32 bits of the corresponding tick value returned here
										                out i_conv_ticks_high,					    // Upper 32 bits of the corresponding tick value returned here
										                meIDS.ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS);   // No flags

	        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    meIOStreamTimeToTicks - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
				
		        return;
	        }

	        meIDS.meIOStreamConfig_t[] arr_stream_config = new meIDS.meIOStreamConfig_t[1];

	        arr_stream_config[0].iChannel = 0;

	        arr_stream_config[0].iStreamConfig = 0;
	        arr_stream_config[0].iRef = meIDS.ME_REF_AI_GROUND;
	        arr_stream_config[0].iFlags = meIDS.ME_IO_STREAM_CONFIG_TYPE_NO_FLAGS;

	        meIDS.meIOStreamTrigger_t stream_trigger = new meIDS.meIOStreamTrigger_t();

	        stream_trigger.iAcqStartTrigType = meIDS.ME_TRIG_TYPE_SW;
        	
	        // int iAcqStartTrigEdge;
	        stream_trigger.iAcqStartTrigChan = meIDS.ME_TRIG_CHAN_DEFAULT;
	        stream_trigger.iAcqStartTicksLow = i_acq_ticks_low;
	        stream_trigger.iAcqStartTicksHigh = i_acq_ticks_high;
	        // int iAcqStartArgs[10];
	        stream_trigger.iScanStartTrigType = meIDS.ME_TRIG_TYPE_TIMER;
	        stream_trigger.iScanStartTicksLow = i_scan_ticks_low;
	        stream_trigger.iScanStartTicksHigh = i_scan_ticks_high;
	        //int iScanStartArgs[10];
	        stream_trigger.iConvStartTrigType = meIDS.ME_TRIG_TYPE_TIMER;
	        stream_trigger.iConvStartTicksLow = i_conv_ticks_low;
	        stream_trigger.iConvStartTicksHigh = i_conv_ticks_high;
	        //int iConvStartArgs[10];
	        // int iScanStopTrigType;
	        // int iScanStopCount;
	        // int iScanStopArgs[10];
	        // int iAcqStopTrigType;
	        // int iAcqStopCount;
	        // int iAcqStopArgs[10];
	        stream_trigger.iFlags = meIDS.ME_IO_STREAM_TRIGGER_TYPE_NO_FLAGS;
        	
	        i_me_error = meIDS.meIOStreamConfig(i_stream_read_device,				    // Device index
									            i_stream_read_subdevice,			    // Subdevice index, 
 									            ref arr_stream_config[0],				    // Pointer to an array of meIOStreamConfig_t structures
									            1,									    // Number of elements in the stream config array above
									            ref stream_trigger,					    // Pointer to an meIOStreamTrigger_t structure
									            2,									    // FIFO IRQ threshold
									            meIDS.ME_IO_STREAM_CONFIG_NO_FLAGS	);	// No flags

	        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    meIOStreamConfig - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
        		
		        return;
	        }

            int i_unit;
            
            double d_phys_min;

	        double d_phys_max;

	        int i_digital_max;
        	
	        i_me_error = meIDS.meQueryRangeInfo(i_stream_read_device,		// Device index
									            i_stream_read_subdevice,	// Subdevice index, 
									            0,							// Range index
									            out i_unit,					// Unit returned here - not require
									            out d_phys_min,				// Physical minimum returned here
									            out d_phys_max,				// Physical maximum returned here
									            out i_digital_max		);	// Digital maximum value returned here

	        System.Console.Write("d_phys_min: {0:F2} d_phys_max: {1:F2} i_digital_max: {2}\n\n", d_phys_min,	d_phys_max,	i_digital_max);
        		
	        System.Console.Write("Press any key to start the acquisition\n\n");
        		
	        System.Console.Write("Press the ESC key at any time to end the acquisition\n\n");
        		
            System.Console.ReadKey(true);
        	
	        meIDS.meIOStreamStart_t[] arr_stream_start = new meIDS.meIOStreamStart_t[1];

	        arr_stream_start[0].iDevice = i_stream_read_device;
	        arr_stream_start[0].iSubdevice = i_stream_read_subdevice;
	        arr_stream_start[0].iStartMode = meIDS.ME_START_MODE_BLOCKING;
	        arr_stream_start[0].iTimeOut = 5;
	        arr_stream_start[0].iFlags = meIDS.ME_IO_STREAM_START_TYPE_NO_FLAGS;
	        arr_stream_start[0].iErrno = 0;
        	
	        i_me_error = meIDS.meIOStreamStart(	ref arr_stream_start[0],			        // Pointer to an array of meIOStreamStart_t structures 
									            1,								        // Number of elements in the stream start array above
									            meIDS.ME_IO_STREAM_START_NO_FLAGS	);	// No flags

	        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    meIOStreamStart - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
        		
		        return;
	        }

	        System.Console.Write("Acquisition started\n\n");

	        while(true)
	        {
                if(Console.KeyAvailable == true)
		        {
                    ConsoleKeyInfo cki = Console.ReadKey(true);

			        if(cki.Key == ConsoleKey.Escape)
			        {
				        break;
			        }
        			
		        }

		        int[] i_buffer = new int[4096];

		        int i_count = 10;
        		
		        i_me_error = meIDS.meIOStreamRead(  i_stream_read_device,			        // Device index
									                i_stream_read_subdevice,		        // Subdevice index, 
									                meIDS.ME_READ_MODE_BLOCKING,			// Read mode 
									                out i_buffer[0],					        // Acquired data values returned in this buffer
									                ref i_count,						    // Size of buffer in data values - Number of data values returned here
									                meIDS.ME_IO_STREAM_READ_NO_FLAGS    );	// No flags

		        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
		        {
                    System.Console.Write("****    meIOStreamRead - Error: {0}    ****\n\n", i_me_error);

                    System.Console.Write("Press any key to terminate\n\n");

                    System.Console.ReadKey(true);

			        break;
		        }

		        for(int index_value = 0; index_value < i_count; index_value++)
		        {
			        double d_volt;
        			
			        i_me_error =  meIDS.meUtilityDigitalToPhysical(	d_phys_min,							// Min physical value
														            d_phys_max,							// Max physical value
														            i_digital_max,						// Maximum digital value
														            i_buffer[index_value],				// Digital value to convert
														            meIDS.ME_MODULE_TYPE_MULTISIG_NONE,	// Module type - not used
														            0.0,								// Reference value - not used
														            out d_volt						);	// Converted physical value returned here

                    System.Console.Write("{0}  -  {1:F2} V\n", i_buffer[index_value], d_volt);
		        }
	        }

	        meIDS.meIOStreamStop_t[] arr_stream_stop = new meIDS.meIOStreamStop_t[1];

	        arr_stream_stop[0].iDevice = i_stream_read_device;
	        arr_stream_stop[0].iSubdevice = i_stream_read_subdevice;
	        arr_stream_stop[0].iStopMode = meIDS.ME_STOP_MODE_IMMEDIATE;
	        arr_stream_stop[0].iFlags = meIDS.ME_IO_STREAM_STOP_TYPE_NO_FLAGS;
	        arr_stream_stop[0].iErrno = 0;

	        i_me_error = meIDS.meIOStreamStop(  ref arr_stream_stop[0],			    // Pointer to an array of meIOStreamStop_t structures  
								                1,								    // Number of elements in the stream stop array above
								                meIDS.ME_IO_STREAM_STOP_NO_FLAGS);	// No flags

	        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
	        {
                System.Console.Write("****    meIOStreamStop - Error: {0}    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

		        meIDS.meClose(0);
        		
		        return;
	        }

	        System.Console.Write("Acquisition ended\n\n");

	        // ---------------------------------------------------------------------
        	

	        meIDS.meClose(0);

            System.Console.Write("Program completed successfully - Press any key to terminate\n\n");

            System.Console.ReadKey(true);
        	
	        return;
        }
    }
}

