// Con_meIOExtIrqCallback.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <stdio.h>
#include <conio.h>

#include <memory.h>

extern "C"
{
#include "..\\medriver.h"
}

#include "..\\TestConsoleGeneric.h"

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

// Forward declaration for callback function

int _stdcall IrqCallback(int iDevice, int iSubdevice, int iChannel, 
							int iIrqCount, int iValue, void *pvContext, int iErrorCode);

int _tmain(int argc, _TCHAR* argv[])
{
	// Call generic console function in parent directory to 
	// open system and list the available devices and subdevices.

	int i_number_of_devices = TestConsoleMEIDSProlog();

	if(i_number_of_devices <= 0)
	{
		return(-1);
	}

	// ---------------------------------------------------------------------
	
	// Count the External IRQ sub-devices in the system

	int i_number_ext_irq_subdevices = 0;

	int i_me_error;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_EXT_IRQ,				// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			++i_number_ext_irq_subdevices;
			
			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}		

	if(i_number_ext_irq_subdevices == 0)
	{
		printf("****    No External IRQ sub-devices found in system   ****\n\n");
		
		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	// Make a list of the External IRQ sub-devices 

	struct SExtIrqSubddevice
	{
		int	m_iDeviceIndex;

		int m_iSubdeviceIndex;

		int m_iNumberOfEdgeTypes;

		int m_arrEdgeTypes[3];
	};

	SExtIrqSubddevice* arrSubdevices = new SExtIrqSubddevice[i_number_ext_irq_subdevices];

	i_number_ext_irq_subdevices = 0;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_EXT_IRQ,				// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			arrSubdevices[i_number_ext_irq_subdevices].m_iDeviceIndex = index_device;

			arrSubdevices[i_number_ext_irq_subdevices].m_iSubdeviceIndex = index_subdevice;

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			int i_number_edge_types = 0;

			if( (i_caps & ME_CAPS_EXT_IRQ_EDGE_RISING) == ME_CAPS_EXT_IRQ_EDGE_RISING )
			{
				arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes[i_number_edge_types] = ME_IRQ_EDGE_RISING;

				++i_number_edge_types;
			}

			if( (i_caps & ME_CAPS_EXT_IRQ_EDGE_FALLING) == ME_CAPS_EXT_IRQ_EDGE_FALLING )
			{
				arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes[i_number_edge_types] = ME_IRQ_EDGE_FALLING;

				++i_number_edge_types;
			}

			if( (i_caps & ME_CAPS_EXT_IRQ_EDGE_ANY) == ME_CAPS_EXT_IRQ_EDGE_ANY )
			{
				arrSubdevices[i_number_ext_irq_subdevices].m_arrEdgeTypes[i_number_edge_types] = ME_IRQ_EDGE_ANY;

				++i_number_edge_types;
			}

			arrSubdevices[i_number_ext_irq_subdevices].m_iNumberOfEdgeTypes = i_number_edge_types; 

			++i_number_ext_irq_subdevices;

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	printf("%d External IRQ sub-devices found:\n\n", i_number_ext_irq_subdevices);

	for(int index_subdevice = 0; index_subdevice < i_number_ext_irq_subdevices; index_subdevice++)
	{
		printf("%2d.   Device: %2d  Sub-device: %2d Supports ",
				index_subdevice + 1,
				arrSubdevices[index_subdevice].m_iDeviceIndex,
				arrSubdevices[index_subdevice].m_iSubdeviceIndex					);

		for(int index_edge_type = 0; index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes; index_edge_type++)
		{
			if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == ME_IRQ_EDGE_RISING)
			{
				printf("Rising edge");
			}
			else if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == ME_IRQ_EDGE_FALLING)
			{
				printf("Falling edge");
			}
			else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == ME_IRQ_EDGE_ANY)
			{
				printf("Any edge");
			}

			if(index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes - 1)
			{
				printf(", ");
			}
			else
			{
				printf("\n");
			}
		}
	}

	printf("\n\nPress any key to continue\n\n");

	getch();

	char buffer[512];

	bool b_terminate = false;
	
	while(true)
	{
		int index_subdevice;

		if(i_number_ext_irq_subdevices > 1)
		{
			while(true)
			{
				printf("Choose Subdevice:\n\n");

				for(index_subdevice = 0; index_subdevice < i_number_ext_irq_subdevices; index_subdevice++)
				{
					printf("%2d.   Device: %2d  Sub-device: %2d Supports ",
							index_subdevice + 1,
							arrSubdevices[index_subdevice].m_iDeviceIndex,
							arrSubdevices[index_subdevice].m_iSubdeviceIndex					);

					for(int index_edge_type = 0; index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes; index_edge_type++)
					{
						if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == ME_IRQ_EDGE_RISING)
						{
							printf("Rising edge");
						}
						else if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == ME_IRQ_EDGE_FALLING)
						{
							printf("Falling edge");
						}
						else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge_type] == ME_IRQ_EDGE_ANY)
						{
							printf("Any edge");
						}

						if(index_edge_type < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes - 1)
						{
							printf(", ");
						}
						else
						{
							printf("\n");
						}
					}
				}

				printf("\nYour choice (1 - %2d) (RETURN to terminate): ", i_number_ext_irq_subdevices);  

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &index_subdevice) != 1 ) )
				{
					continue;
				}

				if( (index_subdevice < 1)||(index_subdevice > i_number_ext_irq_subdevices) )
				{
					continue;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			--index_subdevice;
		}
		else
		{
			index_subdevice = 0;
		}

		int index_edge;

		if(arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes  > 1)
		{
			while(true)
			{
				printf("Choose trigger edge:\n\n");

				for(index_edge = 0; index_edge < arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes; index_edge++)
				{
					printf("    %2d.   ", index_edge + 1);

					if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == ME_IRQ_EDGE_RISING)
					{
						printf("Rising edge\n");
					}
					else if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == ME_IRQ_EDGE_FALLING)
					{
						printf("Falling edge\n");
					}
					else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == ME_IRQ_EDGE_ANY)
					{
						printf("Any edge\n");
					}
				}

				printf("\nYour choice (1 - %2d) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes);  

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &index_edge) != 1 ) )
				{
					continue;
				}

				if( (index_edge < 1)||(index_edge > arrSubdevices[index_subdevice].m_iNumberOfEdgeTypes) )
				{
					continue;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			--index_edge;
		}
		else
		{
			index_edge = 0;
		}


		if(b_terminate)
		{
			break;
		}

        printf(	"Press any key to call:\n\n"
		        "    meIOIrqSetCallback( %-32d      (iDevice)\n"
		        "                        %-32d      (iSubdevice)\n"							
		        "                        %-32s      (pCallback) - See code\n"							
		        "                        %-32s      (*pCallbackContext) Not required\n"							
		        "                        %-32s  );  (iFlags)\n\n",
		        arrSubdevices[index_subdevice].m_iDeviceIndex,
		        arrSubdevices[index_subdevice].m_iSubdeviceIndex,
                "IrqCallback",
                "NULL",
	            "ME_IO_IRQ_SET_CALLBACK_NO_FLAGS"						);


		getch();

		i_me_error = meIOIrqSetCallback(arrSubdevices[index_subdevice].m_iDeviceIndex,					// Device index
										arrSubdevices[index_subdevice].m_iSubdeviceIndex,				// Subdevice index
										IrqCallback,													// Callback routine
										NULL,															// Callback contextz
										ME_IO_IRQ_SET_CALLBACK_NO_FLAGS						);			// Flags


        char* sz_irq_edge;

		printf("Ready to monitor ");

		if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == ME_IRQ_EDGE_RISING)
		{
			printf("Rising edge ");

            sz_irq_edge = "ME_IRQ_EDGE_RISING";
		}
		else if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == ME_IRQ_EDGE_FALLING)
		{
			printf("Falling edge ");

            sz_irq_edge = "ME_IRQ_EDGE_FALLING";
		}
		else // if(arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge] == ME_IRQ_EDGE_ANY)
		{
			printf("Any edge ");

            sz_irq_edge = "ME_IRQ_EDGE_ANY";
		}

		printf("on Device: %d, Subdevice: %d\n\n", 
				arrSubdevices[index_subdevice].m_iDeviceIndex, 
				arrSubdevices[index_subdevice].m_iSubdeviceIndex);

        printf(	"Press any key to call:\n\n"
		        "    meIOIrqStart( %-30d      (iDevice)\n"
		        "                  %-30d      (iSubdevice)\n"							
		        "                  %-30d      (iChannel)\n"							
		        "                  %-30s      (iIrqSource)\n"							
		        "                  %-30s      (iIrqEdge)\n"							
		        "                  %-30s      (iIrqArg)\n"							
		        "                  %-30s  );  (iFlags)\n\n",
		        arrSubdevices[index_subdevice].m_iDeviceIndex,
		        arrSubdevices[index_subdevice].m_iSubdeviceIndex,
                0,
                "ME_IRQ_SOURCE_DIO_LINE",
                sz_irq_edge,
                "ME_VALUE_NOT_USED",
	            "ME_IO_IRQ_START_NO_FLAGS"						);

		printf("and start monitoring external interrupts\n\n");
			
		printf("Press the ESC key at any time to stop monitoring interrupts\n\n");
		
		_getch();
		
		i_me_error =  meIOIrqStart(	arrSubdevices[index_subdevice].m_iDeviceIndex,					// Device index
									arrSubdevices[index_subdevice].m_iSubdeviceIndex,				// Subdevice index
									0,																// Channel index 	
									ME_IRQ_SOURCE_DIO_LINE,											// IRQ source
									arrSubdevices[index_subdevice].m_arrEdgeTypes[index_edge],		// Type of signal edge
									ME_VALUE_NOT_USED,												// Additional argument - Not required here
									ME_IO_IRQ_START_NO_FLAGS									);	// Flags

		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("****    meIOIrqStart - Error: %d    ****\n\n", i_me_error);

			printf("Press any key to terminate\n");
			
			_getch();
			
			meClose(0);
			
			return -1;
		}

		printf("Monitoring interrupts\n\n");

		while(true)
		{
			if( _kbhit() != 0 )
			{
				if(_getch() == 27)
				{
					break;
				}
			}
		}

        printf(	"Calling:\n\n"
		        "    meIOIrqStop( %-30d      (iDevice)\n"
		        "                 %-30d      (iSubdevice)\n"							
		        "                 %-30d      (iChannel)\n"							
		        "                 %-30s  );  (iFlags)\n\n",
		        arrSubdevices[index_subdevice].m_iDeviceIndex,
		        arrSubdevices[index_subdevice].m_iSubdeviceIndex,
                0,
	            "ME_IO_IRQ_STOP_NO_FLAGS"						);

		i_me_error = meIOIrqStop(	arrSubdevices[index_subdevice].m_iDeviceIndex,				// Device index
									arrSubdevices[index_subdevice].m_iSubdeviceIndex,			// Subdevice index
									0,															// Channel index 	
									ME_IO_IRQ_STOP_NO_FLAGS								);		// Flags

	
		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("****    meIOIrqStop - Error: %d    ****\n\n", i_me_error);

			printf("Press any key to terminate\n");
			
			_getch();
		
			meClose(0);
		
			return(-1);
		}

		printf("Stopped monitoring interrupts\n\n");

		printf("Press any key to continue or ESC to exit\n\n");

		if(getch() == 27)
		{
			break;
		}
	}

	// ---------------------------------------------------------------------
	

	meClose(0);
	
	printf("\n\nProgram completed successfully - Press any key to terminate\n\n");

	_getch();
	
	return 0;
}

int _stdcall IrqCallback(int iDevice, int iSubdevice, int iChannel, 
							int iIrqCount, int iValue, void *pvContext, int iErrorCode)
{
	if(iErrorCode == ME_ERRNO_SUCCESS)
	{
		printf("Interrupt: IRQ Count: %d  Value: %d\n", iIrqCount, iValue);
	}
	else
	{
		printf("****    IrqCallback - Error: %d    ****\n\n", iErrorCode);
	}

	return(0);
}

