// Con_meIOSingleWriteAnalogSyncExtTrigger.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <stdio.h>
#include <conio.h>

#include <memory.h>

extern "C"
{
#include "..\\medriver.h"
}

#include "..\\TestConsoleGeneric.h"

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

int _tmain(int argc, _TCHAR* argv[])
{
	// Call generic console function in parent directory to 
	// open system and list the available devices and subdevices.

	int i_number_of_devices = TestConsoleMEIDSProlog();

	if(i_number_of_devices <= 0)
	{
		return(-1);
	}

	// ---------------------------------------------------------------------
	
	// Count the AO sub-devices in the system 

	int i_number_ao_subdevices = 0;

	int i_me_error;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_AO,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			++i_number_ao_subdevices;

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	if(i_number_ao_subdevices == 0)
	{
		printf("****    No AO sub-devices found in system   ****\n\n");
		
		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	};

	// Make a list of the AO sub-devices 

	struct SAORange
	{
		int		m_iUnit;

		double	m_dPhysicalMin;
		double	m_dPhysicalMax;

		int		m_iDigitalMax;
	};

#define MAX_RANGES	 16

	struct SAOSubddevice
	{
		int	m_iDeviceIndex;

		int m_iSubdeviceIndex;

		int m_iNumberOfChannels;

		int m_iNumberOfRanges;

		SAORange m_arrRanges[MAX_RANGES];

		int m_iCurrentRange;
	};

	SAOSubddevice* arrSubdevices = new SAOSubddevice[i_number_ao_subdevices];

	i_number_ao_subdevices = 0;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_AO,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			arrSubdevices[i_number_ao_subdevices].m_iDeviceIndex = index_device;

			arrSubdevices[i_number_ao_subdevices].m_iSubdeviceIndex = index_subdevice;

			int i_number_of_channels;

			i_me_error =  meQueryNumberChannels(index_device,					// Device index
												index_subdevice,				// Subdevice index, 
												&i_number_of_channels	);			// Number of ramges returned here


			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				printf("****    meQueryNumberChannels - Error: %d    ****\n\n", i_me_error);

				printf("Press any key to terminate\n");
				
				_getch();
				
				meClose(0);
				
				return -1;
			}

			arrSubdevices[i_number_ao_subdevices].m_iNumberOfChannels = i_number_of_channels;

			int i_number_of_ranges;

			i_me_error =  meQueryNumberRanges(	index_device,					// Device index
												index_subdevice,				// Subdevice index, 
												ME_UNIT_ANY,					// Unit
												&i_number_of_ranges	);			// Number of ramges returned here


			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				printf("****    meQueryNumberRanges - Error: %d    ****\n\n", i_me_error);

				printf("Press any key to terminate\n");
				
				_getch();
				
				meClose(0);
				
				return -1;
			}

			if(i_number_of_ranges > MAX_RANGES)
			{
				i_number_of_ranges = MAX_RANGES;
			}

			arrSubdevices[i_number_ao_subdevices].m_iNumberOfRanges = i_number_of_ranges;

			for(int index_range = 0; index_range < i_number_of_ranges; index_range++)
			{
				int i_unit;

				double d_phys_min;
				double d_phys_max;

				int i_digital_max;

				i_me_error = meQueryRangeInfo(	index_device,					// Device index
												index_subdevice,				// Subdevice index, 
												index_range,					// Range index
												&i_unit,							// Unit returned here - not require
												&d_phys_min,					// Physical minimum returned here
												&d_phys_max,					// Physical maximum returned here
												&i_digital_max				);	// Digital maximum value returned here

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("****    meQueryRangeInfo - Error: %d    ****\n\n", i_me_error);

					printf("Press any key to terminate\n");
					
					_getch();
					
					meClose(0);
					
					return -1;
				}

				arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_iUnit = i_unit;

				arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_dPhysicalMin = d_phys_min;
				arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_dPhysicalMax = d_phys_max;

				arrSubdevices[i_number_ao_subdevices].m_arrRanges[index_range].m_iDigitalMax = i_digital_max;
			}

			arrSubdevices[i_number_ao_subdevices].m_iCurrentRange = 0;

			++i_number_ao_subdevices;

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	printf("%d AO sub-devices found: in System\n\n", i_number_ao_subdevices);

	for(int index_subdevice = 0; index_subdevice < i_number_ao_subdevices; index_subdevice++)
	{
		printf("%2d.   Device: %2d  Sub-device: %2d  Channels: %2d  Ranges: %2d\n",
				index_subdevice + 1,
				arrSubdevices[index_subdevice].m_iDeviceIndex,
				arrSubdevices[index_subdevice].m_iSubdeviceIndex,
				arrSubdevices[index_subdevice].m_iNumberOfChannels,
				arrSubdevices[index_subdevice].m_iNumberOfRanges		);
	}

	printf("Press any key to continue\n");

	getch();

	printf("\n\n");

	char buffer[512];

	bool b_terminate = false;
	
	while(true)
	{
		int index_subdevice;

		if(i_number_ao_subdevices > 1)
		{
			while(true)
			{
				printf("Choose Subdevice:\n\n");

				for(index_subdevice = 0; index_subdevice < i_number_ao_subdevices; index_subdevice++)
				{
					printf("%2d.   Device: %2d  Sub-device: %2d  Channels: %2d  Ranges: %2d\n",
					index_subdevice + 1,
					arrSubdevices[index_subdevice].m_iDeviceIndex,
					arrSubdevices[index_subdevice].m_iSubdeviceIndex,
					arrSubdevices[index_subdevice].m_iNumberOfChannels,
					arrSubdevices[index_subdevice].m_iNumberOfRanges		);
				}

				printf("\nYour choice (1 - %2d) (RETURN to terminate): ", i_number_ao_subdevices); 

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &index_subdevice) != 1 ) )
				{
					continue;
				}

				if( (index_subdevice < 1)||(index_subdevice > i_number_ao_subdevices) )
				{
					continue;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			--index_subdevice;
		}
		else
		{
			index_subdevice = 0;
		}

		bool b_config = true;

		while(true)
		{
			printf("Choose Function:\n\n");

			printf(	"C) meIOSingleConfig\n"
					"W) meIOSingle (Write)\n"	);

			printf("\nYour choice (C/W) (RETURN to terminate): "); 

			char ch = getche();

			printf("\n\n");
			
			if(ch == 13)
			{
				b_terminate = true;
				
				break;
			}
			
			if( (ch == 'c')||(ch == 'C') )
			{
				b_config = true;
				
				break;;
			}
			else if( (ch == 'w')||(ch == 'W') )
			{
				b_config = false;
				
				break;;
			}
		}

		if(b_terminate)
		{
			break;
		}

		int index_channel;

		if(arrSubdevices[index_subdevice].m_iNumberOfChannels > 1)
		{
			while(true)
			{
				printf("Channel number (0 - %2d) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfChannels - 1); 

				gets(&buffer[0]);

				printf("\n\n");

				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &index_channel) != 1 ) )
				{
					continue;
				}

				if( (index_channel < 0)||(index_channel >= arrSubdevices[index_subdevice].m_iNumberOfChannels) )
				{
					continue;
				}
				
				break;
			}
		
			if(b_terminate)
			{
				break;
			}
		}
		else
		{
			index_channel = 0;
		}

		if(b_config)
		{
			int index_range;

			if(arrSubdevices[index_subdevice].m_iNumberOfRanges > 1)
			{
				while(true)
				{
					printf("Choose Range:\n\n");

					for(index_range = 0; index_range < arrSubdevices[index_subdevice].m_iNumberOfRanges; index_range++)
					{
						printf("%2d.   %8.3f - %8.3f %s\n",
						index_range + 1,
						arrSubdevices[index_subdevice].m_arrRanges[index_range].m_dPhysicalMin,
						arrSubdevices[index_subdevice].m_arrRanges[index_range].m_dPhysicalMax,
						( (arrSubdevices[index_subdevice].m_arrRanges[index_range].m_iUnit == ME_UNIT_VOLT) ? "  Volt" : "Ampere" ) );
					}
					
					printf("\nYour choice (1 - %2d) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfRanges);  

					gets(&buffer[0]);

					printf("\n\n");

					if(buffer[0] == 0)
					{
						b_terminate = true;
						
						break;
					}
					
					if( (sscanf(&buffer[0], "%d", &index_range) != 1 ) )
					{
						continue;
					}

					if( (index_range < 1)||(index_range > arrSubdevices[index_subdevice].m_iNumberOfRanges) )
					{
						continue;
					}
					
					break;
				}

				if(b_terminate)
				{
					break;
				}

				--index_range;
			}
			else
			{
				index_range = 0;
			}

			int i_trigger_channel = ME_TRIG_CHAN_DEFAULT;

			while(true)
			{
				printf("Choose Trigger Channel:\n\n");

				printf(	"D)efault\n"
						"S)ynchronous\n"	);

				printf("\nYour choice (D/S) (RETURN to terminate): "); 

				char ch = getche();

				printf("\n\n");
				
				if(ch == 13)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (ch == 'd')||(ch == 'D') )
				{
					i_trigger_channel = ME_TRIG_CHAN_DEFAULT;
					
					break;;
				}
				else if( (ch == 's')||(ch == 'S') )
				{
					i_trigger_channel = ME_TRIG_CHAN_SYNCHRONOUS;
					
					break;;
				}
			}

			if(b_terminate)
			{
				break;
			}

			int i_trigger_type = ME_TRIG_TYPE_SW;
		
			while(true)
			{
				printf("Choose Trigger Type:\n\n");

				printf(	"S)oftware\n"
						"E)xternal Digital\n"	);

				printf("\nYour choice (S/E) (RETURN to terminate): "); 

				char ch = getche();

				printf("\n\n");
				
				if(ch == 13)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (ch == 's')||(ch == 'S') )
				{
					i_trigger_type = ME_TRIG_TYPE_SW;
					
					break;;
				}
				else if( (ch == 'e')||(ch == 'E') )
				{
					i_trigger_type = ME_TRIG_TYPE_EXT_DIGITAL;
					
					break;;
				}
			}

			if(b_terminate)
			{
				break;
			}

			int i_trigger_edge = ME_VALUE_NOT_USED;

			if(i_trigger_type == ME_TRIG_TYPE_EXT_DIGITAL)
			{
				i_trigger_edge = ME_TRIG_EDGE_ANY;

				while(true)
				{
					printf("Choose Trigger Edge:\n\n");

					printf(	"R)ising\n"
							"F)alling\n"
							"A)ny\n"	);

					printf("\nYour choice (R/F/A) (RETURN to terminate): "); 

					char ch = getche();

					printf("\n\n");
					
					if(ch == 13)
					{
						b_terminate = true;
						
						break;
					}
					
					if( (ch == 'r')||(ch == 'R') )
					{
						i_trigger_edge = ME_TRIG_EDGE_RISING;
						
						break;;
					}
					if( (ch == 'f')||(ch == 'F') )
					{
						i_trigger_edge = ME_TRIG_EDGE_FALLING;
						
						break;;
					}
					if( (ch == 'a')||(ch == 'A') )
					{
						i_trigger_edge = ME_TRIG_EDGE_ANY;
						
						break;;
					}
				}

				if(b_terminate)
				{
					break;
				}

			}
		
			// Configure the device

			printf("Press any key to configure the sub-device\n");

			getch();

			i_me_error = meIOSingleConfig(	arrSubdevices[index_subdevice].m_iDeviceIndex,			// Device index
											arrSubdevices[index_subdevice].m_iSubdeviceIndex,		// Subdevice index, 
											index_channel,											// Channel index
											index_range,											// Range index
											ME_REF_AO_GROUND,										// Reference
											i_trigger_channel,										// Trigger channel - standard
											i_trigger_type,											// Trigger type - software
											i_trigger_edge,											// Trigger edge - not applicable
											ME_IO_SINGLE_CONFIG_NO_FLAGS						);	// Flags

			if(i_me_error == ME_ERRNO_SUCCESS)
			{
				printf("\nsub-device configured\n\n", i_me_error);

				arrSubdevices[index_subdevice].m_iCurrentRange = index_range;
			}
			else 
			{
				printf("\n****    meIOSingleConfig - Error: %d    ****\n\n", i_me_error);
			}

			printf("Press any key to continue\n");

			getch();

			printf("\n\n");
		}
		else // if(!b_config)
		{
			// Fetch the range information which will be required to convert from 
			// physical units to a digital value for the output below

			double d_phys_min;

			double d_phys_max;

			int i_digital_max;
			
			i_me_error = meQueryRangeInfo(	arrSubdevices[index_subdevice].m_iDeviceIndex,			// Device index
											arrSubdevices[index_subdevice].m_iSubdeviceIndex,		// Subdevice index, 
											arrSubdevices[index_subdevice].m_iCurrentRange,			// Range index
											NULL,													// Unit returned here - not require
											&d_phys_min,											// Physical minimum returned here
											&d_phys_max,											// Physical maximum returned here
											&i_digital_max										);	// Digital maximum value returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				printf("****    meQueryRangeInfo - Error: %d    ****\n\n", i_me_error);

				printf("Press any key to terminate\n");
				
				_getch();
				
				meClose(0);
				
				return -1;
			}

			int i_digital_value;

			double d_value;

			char buffer[128];

			while(true)
			{
				printf("Output value V (%5.2lf  - %5.2lf) (RETURN to terminate): ", d_phys_min, d_phys_max);

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;

					break;
				}
				
				if( (sscanf(&buffer[0], "%lf", &d_value) != 1 ) )
				{
					continue;
				}

				if( (d_value < d_phys_min)||(d_phys_max < d_value) )
				{
					continue;
				}

				i_me_error = meUtilityPhysicalToDigital(d_phys_min,				// Minimum physiacal value
														d_phys_max,				// Maximum physical value
														i_digital_max,			// Maximum digital value
														d_value,				// Physical value to convert
														&i_digital_value	);	// Corresponding digital value for meIOSingle returned here

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("****    meUtilityPhysicalToDigital - Error: %d    ****\n\n", i_me_error);

					printf("Press any key to terminate\n");
					
					_getch();
					
					meClose(0);
					
					return -1;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			int i_timeout_ms;

			while(true)
			{
				printf("Timeout milliseconds (0  - 10000, 0 --> No Timeout) (RETURN to terminate): ");

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;

					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &i_timeout_ms) != 1 ) )
				{
					continue;
				}

				if( (i_timeout_ms < 0)||(10000 < i_timeout_ms) )
				{
					continue;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			int i_flags = ME_IO_SINGLE_TYPE_NO_FLAGS;

			while(true)
			{
				printf(	"Synchronous Start? Y/N (RETURN to terminate): ");

				char ch = getche();

				printf("\n\n");
				
				if(ch == 13)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (ch == 'y')||(ch == 'Y') )
				{
					i_flags|= ME_IO_SINGLE_TYPE_TRIG_SYNCHRONOUS;
					
					break;;
				}
				else if( (ch == 'n')||(ch == 'N') )
				{
					break;;
				}
			}

			if(b_terminate)
			{
				break;
			}

			while(true)
			{
				printf(	"Blocking Mode? Y/N (RETURN to terminate): ");

				char ch = getche();

				printf("\n\n");
				
				if(ch == 13)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (ch == 'y')||(ch == 'Y') )
				{
					break;
				}
				else if( (ch == 'n')||(ch == 'N') )
				{
					i_flags|= ME_IO_SINGLE_TYPE_WRITE_NONBLOCKING;
					
					break;;
				}
			}

			if(b_terminate)
			{
				break;
			}

			printf("Press any key to write to subdevice\n");

			getch();

			printf("\n");

			meIOSingle_t io_single[1];

			io_single[0].iDevice = arrSubdevices[index_subdevice].m_iDeviceIndex;

			io_single[0].iSubdevice = arrSubdevices[index_subdevice].m_iSubdeviceIndex;
			io_single[0].iChannel	= index_channel;
			io_single[0].iDir		= ME_DIR_OUTPUT;
			io_single[0].iValue		= i_digital_value;
			io_single[0].iTimeOut	= i_timeout_ms;							// No timeout - not required for software output
			io_single[0].iFlags		= i_flags;
			
			i_me_error = meIOSingle(&io_single[0],				// Output list 
									1,							// Number of elements in the above list 
									ME_IO_SINGLE_NO_FLAGS	);	// Flags

			if(i_me_error == ME_ERRNO_SUCCESS)
			{
				printf(	"Output %5.2lf V (%5d) to Device: %2d  Sub-device: %2d\n\n\n", 
						d_value, 
						i_digital_value,
						arrSubdevices[index_subdevice].m_iDeviceIndex,
						arrSubdevices[index_subdevice].m_iSubdeviceIndex					);
			}
			else if(i_me_error == ME_ERRNO_TIMEOUT)
			{
				printf("****    meIOSingle - Timeout    ****\n\n", i_me_error);
			}
			else
			{
				printf("****    meIOSingle - Error: %d    ****\n\n", i_me_error);

				printf("Press any key to terminate\n");
				
				_getch();
				
				meClose(0);
				
				return -1;
			}

			printf("Press any key to continue\n");

			getch();

			printf("\n\n");
		}
	}

	// ---------------------------------------------------------------------
	
	meClose(0);
	
	printf("Program completed successfully - Press any key to terminate\n\n");

	_getch();
	
	return 0;
}

