// Con_meIOStreamReadDigital.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <stdio.h>
#include <conio.h>

#include <memory.h>

#include <windows.h>

extern "C"
{
#include "..\\medriver.h"
}

#include "..\\TestConsoleGeneric.h"

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

int iBuffer[0xFFFF];

int _tmain(int argc, _TCHAR* argv[])
{
	// Call generic console function in parent directory to 
	// open system and list the available devices and subdevices.

	int i_number_of_devices = TestConsoleMEIDSProlog();

	if(i_number_of_devices <= 0)
	{
		return(-1);
	}

	// ---------------------------------------------------------------------
	
	int i_stream_write_device	= -1;
	int i_stream_write_subdevice = -1;

	// Search for an DI-Streaming subdevice in the system

	int i_me_error;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		i_me_error = meQuerySubdeviceByType(index_device,					// Device
											0,								// Begin search at this sub-device 
											ME_TYPE_DO,						// Type of sub-device to search for
											ME_SUBTYPE_STREAMING,			// Sub-type of sub-device to search for
											&i_stream_write_subdevice	);	// sub-device index returned here

		if(i_me_error == ME_ERRNO_SUCCESS)
		{
			i_stream_write_device = index_device;

			break;
		}
	}

	if(i_stream_write_device == -1)
	{
		printf("****    No DO-Streaming sub-device found in system    ****\n\n");
		
		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	printf("Device: %d, Subdevice: %d is a DO-Streaming device\n\n", i_stream_write_device, i_stream_write_subdevice);

	char buffer[512];

	double d_required_freq = 500000.0;

	bool b_terminate = false;

	while(true)
	{
		printf("Frequency Hz (1 - 33000000) (RETURN to terminate): ");

		gets(&buffer[0]);

		printf("\n\n");
		
		if(buffer[0] == 0)
		{
			b_terminate = true;
			
			break;
		}
		
		if( (sscanf(&buffer[0], "%lf", &d_required_freq) != 1 ) )
		{
			continue;
		}

		if( (d_required_freq < 1.0)||(33000000.0 < d_required_freq) )
		{
			continue;
		}

		break;
	}

	if(b_terminate)
	{
		printf("****    Program aborted    ****\n\n");

		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	int i_acq_ticks_low = 0;
	int i_acq_ticks_high = 0;

	int i_scan_ticks_low = 0;
	int i_scan_ticks_high = 0;

	double d_achieved_freq = d_required_freq;

	int i_conv_ticks_low = 0;
	int i_conv_ticks_high = 0;

	i_me_error = meIOStreamFrequencyToTicks(i_stream_write_device,				// Device index
											i_stream_write_subdevice,			// Subdevice indexe, 
											ME_TIMER_CONV_START,				// Timer used 
											&d_achieved_freq,					// Required frequency Hz, Achieved frequency returned here	 
											&i_conv_ticks_low,					// Lower 32 bits of the corresponding tick value returned here
											&i_conv_ticks_high,					// Upper 32 bits of the corresponding tick value returned here
											ME_IO_FREQUENCY_TO_TICKS_NO_FLAGS); // No flags

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("****    meIOStreamFrequencyToTicks - Error: %d    ****\n\n", i_me_error);

		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	printf("meIOStreamFrequencyToTicks - Required Freq: %f Achieved Freq: %f Ticks low: %d Ticks high: %d\n\n", d_required_freq, d_achieved_freq, i_conv_ticks_low, i_conv_ticks_high);

	if(b_terminate)
	{
		printf("****    Program aborted    ****\n\n");

		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	meIOStreamConfig_t arr_stream_config[1];

	arr_stream_config[0].iChannel = 0;

	arr_stream_config[0].iStreamConfig = 0;
	arr_stream_config[0].iRef = ME_REF_NONE;
	arr_stream_config[0].iFlags = ME_IO_STREAM_CONFIG_TYPE_NO_FLAGS;

	meIOStreamTrigger_t stream_trigger;

	memset( &stream_trigger, 0, sizeof(meIOStreamTrigger_t) );

	stream_trigger.iAcqStartTrigType = ME_TRIG_TYPE_SW;
	
	// int iAcqStartTrigEdge;

	stream_trigger.iAcqStartTrigChan = ME_TRIG_CHAN_DEFAULT;
	stream_trigger.iAcqStartTicksLow = i_acq_ticks_low;
	stream_trigger.iAcqStartTicksHigh = i_acq_ticks_high;

	// int iAcqStartArgs[10];

	stream_trigger.iScanStartTrigType = ME_TRIG_TYPE_FOLLOW;
	stream_trigger.iScanStartTicksLow = i_scan_ticks_low;
	stream_trigger.iScanStartTicksHigh = i_scan_ticks_high;

	//int iScanStartArgs[10];

	stream_trigger.iConvStartTrigType = ME_TRIG_TYPE_TIMER;
	stream_trigger.iConvStartTicksLow = i_conv_ticks_low;
	stream_trigger.iConvStartTicksHigh = i_conv_ticks_high;

	// int iConvStartArgs[10];
	// int iScanStopTrigType;
	// int iScanStopCount;
	// int iScanStopArgs[10];
	// int iAcqStopTrigType;
	// int iAcqStopCount;
	// int iAcqStopArgs[10];

	stream_trigger.iFlags = ME_IO_STREAM_TRIGGER_TYPE_NO_FLAGS;

	i_me_error = meIOStreamConfig(	i_stream_write_device,				// Device index
									i_stream_write_subdevice,			// Subdevice index, 
 									&arr_stream_config[0],				// Pointer to an array of meIOStreamConfig_t structures
									1,									// Number of elements in the stream config array above
									&stream_trigger,					// Pointer to an meIOStreamTrigger_t structure
									0,									// Not used here
									0x80000000					);		// Temporary flag for Tests
	
	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("****    meIOStreamConfig - Error: %d    ****\n\n", i_me_error);

		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	printf("Press any key to start the acquisition\n\n");
		
	printf("Press the ESC key at any time to end the acquisition\n\n");
		
	_getch();

	for(int index = 0; index < 0x10000; index++)
	{
		iBuffer[index] = index;
	}

	int i_write_size = 0x10000;
	
	i_me_error = meIOStreamWrite(	i_stream_write_device,			// Device index
									i_stream_write_subdevice,		// Subdevice index, 
									ME_WRITE_MODE_PRELOAD,			// Read mode 
									&iBuffer[0],					// Acquired data values returned in this buffer
									&i_write_size,					// Size of buffer in data values - Number of data values returned here
									ME_IO_STREAM_READ_NO_FLAGS);	// No flags

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("****    meIOStreamWrite - Error: %d    ****\n\n", i_me_error);

		printf("Press any key to end the acquisition\n");
	
		_getch();

		return -1;
	}

	meIOStreamStart_t arr_stream_start[1];

	arr_stream_start[0].iDevice = i_stream_write_device;
	arr_stream_start[0].iSubdevice = i_stream_write_subdevice;
	arr_stream_start[0].iStartMode = ME_START_MODE_BLOCKING;
	arr_stream_start[0].iTimeOut = 5;
	arr_stream_start[0].iFlags = ME_IO_STREAM_START_TYPE_NO_FLAGS;
	arr_stream_start[0].iErrno = 0;
	
	i_me_error = meIOStreamStart(	&arr_stream_start[0],			// Pointer to an array of meIOStreamStart_t structures 
									1,								// Number of elements in the stream start array above
									ME_IO_STREAM_START_NO_FLAGS	);	// No flags

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("****    meIOStreamStart - Error: %d    ****\n\n", i_me_error);

		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	unsigned long ul_start_ms = GetTickCount();

	printf("Acquisition started\n\n");

	unsigned short us_last_value = 0;

	int b_first_value = 1;

	int i_total_data = 0;

	int i_data_errors = 0;

	while(true)
	{
		if( _kbhit() != 0 )
		{
			if(_getch() == 27)
			{
				break;
			}
		}

		i_write_size = 0x10000;
		
		i_me_error = meIOStreamWrite(	i_stream_write_device,			// Device index
										i_stream_write_subdevice,		// Subdevice index, 
										ME_WRITE_MODE_BLOCKING,			// Read mode 
										&iBuffer[0],					// Acquired data values returned in this buffer
										&i_write_size,					// Size of buffer in data values - Number of data values returned here
										ME_IO_STREAM_READ_NO_FLAGS);	// No flags

		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("****    meIOStreamReadDigital - Error: %d    ****\n\n", i_me_error);

			printf("Press any key to end the acquisition\n");
		
			_getch();

			break;
		}

		unsigned long ul_ms = GetTickCount() - ul_start_ms;

		i_total_data+= i_write_size;

		printf("%d data sent in %8.3f seconds\n", i_total_data, (double)ul_ms / 1000.0);
	}

	meIOStreamStop_t arr_stream_stop[1];

	arr_stream_stop[0].iDevice = i_stream_write_device;
	arr_stream_stop[0].iSubdevice = i_stream_write_subdevice;
	arr_stream_stop[0].iStopMode = ME_STOP_MODE_IMMEDIATE;
	arr_stream_stop[0].iFlags = ME_IO_STREAM_STOP_TYPE_NO_FLAGS;
	arr_stream_stop[0].iErrno = 0;

	i_me_error = meIOStreamStop(&arr_stream_stop[0],			// Pointer to an array of meIOStreamStop_t structures  
								1,								// Number of elements in the stream stop array above
								ME_IO_STREAM_STOP_NO_FLAGS);	// No flags

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("****    meIOStreamStop - Error: %d    ****\n\n", i_me_error);

		printf("Press any key to terminate\n");
		
		_getch();
	
		meClose(0);
	
		return(-1);
	}

	printf("Acquisition ended\n\n");

	// ---------------------------------------------------------------------
	

	meClose(0);
	
	printf("Program completed successfully - Press any key to terminate\n\n");

	_getch();
	
	return 0;
}

