//==============================================================================
//Copyright(c)  Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : medriver.h
//              function prototypes of ME-iDS import library
//==============================================================================

#ifndef _MEDRIVER_H_
#define _MEDRIVER_H_

#include "metypes.h"
#include "meerror.h"
#include "medefines.h"

#ifdef __cplusplus
extern "C" {
#endif

    /*===========================================================================
      Functions to access the driver system
      =========================================================================*/

    int meOpen(int iFlags);
    int meClose(int iFlags);

    int meLockDriver(int iLock, int iFlags);
    int meLockDevice(int iDevice, int iLock, int iFlags);
    int meLockSubdevice(int iDevice, int iSubdevice, int iLock, int iFlags);

    /*===========================================================================
      Error handling functions
      =========================================================================*/

    int meErrorGetLast(int* piErrorCode, int iFlags);
    int meErrorGetLastMessage(char *pcErrorMsg, int iCount);
    int meErrorGetMessage(int iErrorCode, char *pcErrorMsg, int iCount);
    int meErrorSetDefaultProc(int iSwitch);
    int meErrorSetUserProc(meErrorCB_t pErrorProc);


    /*===========================================================================
      Functions to perform I/O on a device
      =========================================================================*/

    int meIOIrqSetCallback(
            int iDevice,
            int iSubdevice,
            meIOIrqCB_t pCallback,
            void *pCallbackContext,
            int iFlags);
    int meIOIrqStart(
            int iDevice,
            int iSubdevice,
            int iChannel,
            int iIrqSource,
            int iIrqEdge,
            int iIrqArg,
            int iFlags);
    int meIOIrqStop(
            int iDevice,
            int iSubdevice,
            int iChannel,
            int iFlags);
    int meIOIrqWait(
            int iDevice,
            int iSubdevice,
            int iChannel,
            int *piIrqCount,
            int *piValue,
            int iTimeOut,
            int iFlags);

    int meIOResetDevice(int iDevice, int iFlags);
    int meIOResetSubdevice(int iDevice, int iSubdevice, int iFlags);

    int meIOSetChannelOffset(
            int iDevice,
            int iSubdevice,
            int iChannel,
            int iRange,
            double *pdOffset,
            int iFlags);
    
    int meIOSingleConfig(
            int iDevice,
            int iSubdevice,
            int iChannel,
            int iSingleConfig,
            int iRef,
            int iTrigChan,
            int iTrigType,
            int iTrigEdge,
            int iFlags);
    
    int meIOSingle(meIOSingle_t *pSingleList, int iCount, int iFlags);
    
    int meIOSingleTimeToTicks(
            int iDevice,
            int iSubdevice,
            int iTimer,
            double *pdTime,
            int *piTicksLow,
            int *piTicksHigh,
            int iFlags);
    int meIOSingleTicksToTime(
            int iDevice,
            int iSubdevice,
            int iTimer,
            int iTicksLow,
            int iTicksHigh,
            double *pdTime,
            int iFlags);

    int meIOStreamConfig(
            int iDevice,
            int iSubdevice,
            meIOStreamConfig_t *pConfigList,
            int iCount,
            meIOStreamTrigger_t *pTrigger,
            int iFifoIrqThreshold,
            int iFlags);
    int meIOStreamNewValues(
            int iDevice,
            int iSubdevice,
            int iTimeOut,
            int *piCount,
            int iFlags);
    int meIOStreamRead(
            int iDevice,
            int iSubdevice,
            int iReadMode,
            int *piValues,
            int *piCount,
            int iFlags);
    int meIOStreamWrite(
            int iDevice,
            int iSubdevice,
            int iWriteMode,
            int *piValues,
            int *piCount,
            int iFlags);
    int meIOStreamStart(meIOStreamStart_t *pStartList, int iCount, int iFlags);
    int meIOStreamStop(meIOStreamStop_t *pStopList, int iCount, int iFlags);
    int meIOStreamStatus(
            int iDevice,
            int iSubdevice,
            int iWait,
            int *piStatus,
            int *piCount,
            int iFlags);
    int meIOStreamSetCallbacks(
            int iDevice,
            int iSubdevice,
            meIOStreamCB_t pStartCB,
            void *pStartCBContext,
            meIOStreamCB_t pNewValuesCB,
            void *pNewValuesCBContext,
            meIOStreamCB_t pEndCB,
            void *pEndCBContext,
            int iFlags);
    int meIOStreamFrequencyToTicks(
            int iDevice,
            int iSubdevice,
            int iTimer,
            double *pdFrequency,
            int *piTicksLow,
            int *piTicksHigh,
            int iFlags);
    int meIOStreamTimeToTicks(
            int iDevice,
            int iSubdevice,
            int iTimer,
            double *pdTime,
            int *piTicksLow,
            int *piTicksHigh,
            int iFlags);


    /*===========================================================================
      Functions to query the driver system
      =========================================================================*/

    int meQueryDescriptionDevice(int iDevice, char *pcDescription, int iCount);

    int meQueryInfoDevice(
            int iDevice,
            int *piVendorId,
            int *piDeviceId,
            int *piSerialNo,
            int *piBusType,
            int *piBusNo,
            int *piDevNo,
            int *piFuncNo,
            int *piPlugged);

    int meQueryNameDevice(int iDevice, char *pcName, int iCount);
    int meQueryNameDeviceDriver(int iDevice, char *pcName, int iCount);

    int meQueryNumberDevices(int *piNumber);
    int meQueryNumberSubdevices(int iDevice, int *piNumber);
    int meQueryNumberChannels(int iDevice, int iSubdevice, int *piNumber);
    int meQueryNumberRanges(
            int iDevice,
            int iSubdevice,
            int iUnit,
            int *piNumber);

    int meQueryRangeByMinMax(
            int iDevice,
            int iSubdevice,
            int iUnit,
            double *pdMin,
            double *pdMax,
            int *piMaxData,
            int *piRange);
    int meQueryRangeInfo(
            int iDevice,
            int iSubdevice,
            int iRange,
            int *piUnit,
            double *pdMin,
            double *pdMax,
            int *piMaxData);

    int meQuerySubdeviceByType(
            int iDevice,
            int iStartSubdevice,
            int iType,
            int iSubtype,
            int *piSubdevice);
    int meQuerySubdeviceType(
            int iDevice,
            int iSubdevice,
            int *piType,
            int *piSubtype);
    int meQuerySubdeviceCaps(
            int iDevice,
            int iSubdevice,
            int *piCaps);
    int meQuerySubdeviceCapsArgs(
            int iDevice,
            int iSubdevice,
            int iCap,
            int *piArgs,
            int iCount);

    int meQueryVersionLibrary(int *piVersion);
    int meQueryVersionMainDriver(int *piVersion);
    int meQueryVersionDeviceDriver(int iDevice, int *piVersion);

    /*===========================================================================
      Common utility functions
      =========================================================================*/

    int meUtilityExtractValues(
            int iChannel,
            int *piAIBuffer,
            int iAIBufferCount,
            meIOStreamConfig_t *pConfigList,
            int iConfigListCount,
            int *piChanBuffer,
            int *piChanBufferCount);
    int meUtilityDigitalToPhysical(
            double dMin,
            double dMax,
            int iMaxData,
            int iData,
            int iModuleType,
            double dRefValue,
            double *pdPhysical);
    int meUtilityDigitalToPhysicalV(
            double dMin,
            double dMax,
            int iMaxData,
            int *piDataBuffer,
            int iCount,
            int iModuleType,
            double dRefValue,
            double *pdPhysicalBuffer);
    int meUtilityPhysicalToDigital(
            double dMin,
            double dMax,
            int iMaxData,
            double dPhysical,
            int *piData);
    int meUtilityPhysicalToDigitalV(
            double dMin,
            double dMax,
            int iMaxData,
            double* pdPhysicalBuffer,
            int iCount,
            int *piDataBuffer);
    int meUtilityPWMStart(
            int iDevice,
            int iSubdevice1,
            int iSubdevice2,
            int iSubdevice3,
            int iRef,
            int iPrescaler,
            int iDutyCycle,
            int iFlag);
    int meUtilityPWMRestart(
            int iDevice,
            int iSubdevice1,
            int iRef,
            int iPrescaler);
    int meUtilityPWMStop(int iDevice,
            int iSubdevice1);

    /*===========================================================================
      ANSI property routines
      =========================================================================*/

    int mePropertyGetIntA(char* pcPropertyPath, int* piPropertyValue);

    int mePropertyGetDoubleA(char* pcPropertyPath, double* pdPropertyValue);

    int mePropertyGetStringA(char* pcPropertyPath, char* pcPropertyValue, int iBufferLength);

    int mePropertySetIntA(char* pcPropertyPath, int iPropertyValue);

    int mePropertySetDoubleA(char* pcPropertyPath, double dPropertyValue);

    int mePropertySetStringA(char* pcPropertyPath, char* pcPropertyValue);

    
    /*===========================================================================
      Wide character property routines
      =========================================================================*/

    int mePropertyGetIntW(wchar_t* pwcPropertyPath, int* piPropertyValue);

    int mePropertyGetDoubleW(wchar_t* pwcPropertyPath, double* pdPropertyValue);

	int mePropertyGetStringW(wchar_t* pwcPropertyPath, wchar_t* pwcPropertyValue, int iBufferLength);

    int mePropertySetIntW(wchar_t* pwcPropertyPath, int iPropertyValue);

    int mePropertySetDoubleW(wchar_t* pwcPropertyPath, double dPropertyValue);

    int mePropertySetStringW(wchar_t* pwcPropertyPath, wchar_t* pwcPropertyValue);

#ifdef __cplusplus
}
#endif

#endif
